import io
import json
import os
import platform
import requests
from .input_file import InputFile
from .exception import RilzException
from .encoders.value_class_encoder import ValueClassEncoder

class Client:
    def __init__(self):
        self._chunk_size = 5*1024*1024
        self._self_signed = False
        self._endpoint = 'https://rilck.com/v1'
        self._global_headers = {
            'content-type': '',
            'user-agent' : f'RilzPythonSDK/11.0.0 ({platform.uname().system}; {platform.uname().version}; {platform.uname().machine})',
            'x-sdk-name': 'Python',
            'x-sdk-platform': 'server',
            'x-sdk-language': 'python',
            'x-sdk-version': '11.0.0',
            'X-Rilz-Response-Format' : '1.7.0',
        }

    def set_self_signed(self, status=True):
        self._self_signed = status
        return self

    def set_endpoint(self, endpoint):
        if not endpoint.startswith('http://') and not endpoint.startswith('https://'):
            raise RilzException('Invalid endpoint URL: ' + endpoint)

        self._endpoint = endpoint
        return self

    def add_header(self, key, value):
        self._global_headers[key.lower()] = value
        return self

    def set_project(self, value):
        """Your project ID"""

        self._global_headers['x-appwrite-project'] = value
        return self

    def set_key(self, value):
        """Your secret API key"""

        self._global_headers['x-appwrite-key'] = value
        return self

    def set_jwt(self, value):
        """Your secret JSON Web Token"""

        self._global_headers['x-appwrite-jwt'] = value
        return self

    def set_locale(self, value):
        self._global_headers['x-appwrite-locale'] = value
        return self

    def set_session(self, value):
        """The user session to authenticate with"""

        self._global_headers['x-appwrite-session'] = value
        return self

    def set_forwarded_user_agent(self, value):
        """The user agent string of the client that made the request"""

        self._global_headers['x-forwarded-user-agent'] = value
        return self

    def call(self, method, path='', headers=None, params=None, response_type='json'):
        if headers is None:
            headers = {}

        if params is None:
            params = {}

        params = {k: v for k, v in params.items() if v is not None}  # Remove None values from params dictionary

        data = {}
        files = {}
        stringify = False

        headers = {**self._global_headers, **headers}

        if method != 'get':
            data = params
            params = {}

        if headers['content-type'].startswith('application/json'):
            data = json.dumps(data, cls=ValueClassEncoder)

        if headers['content-type'].startswith('multipart/form-data'):
            del headers['content-type']
            stringify = True
            for key in data.copy():
                if isinstance(data[key], InputFile):
                    files[key] = (data[key].filename, data[key].data)
                    del data[key]
            data = self.flatten(data, stringify=stringify)

        response = None
        try:
            response = requests.request(  # call method dynamically https://stackoverflow.com/a/4246075/2299554
                method=method,
                url=self._endpoint + path,
                params=self.flatten(params, stringify=stringify),
                data=data,
                files=files,
                headers=headers,
                verify=(not self._self_signed),
                allow_redirects=False if response_type == 'location' else True
            )

            response.raise_for_status()

            warnings = response.headers.get('x-appwrite-warning')
            if warnings:
                for warning in warnings.split(';'):
                    print(f'Warning: {warning}')

            content_type = response.headers['Content-Type']

            if response_type == 'location':
                return response.headers.get('Location')

            if content_type.startswith('application/json'):
                return response.json()

            return response._content
        except Exception as e:
            if response != None:
                content_type = response.headers['Content-Type']
                if content_type.startswith('application/json'):
                    raise RilzException(response.json()['message'], response.status_code, response.json().get('type'), response.text)
                else:
                    raise RilzException(response.text, response.status_code, None, response.text)
            else:
                raise RilzException(e)

    def chunked_upload(
        self,
        path,
        headers = None,
        params = None,
        param_name = '',
        on_progress = None,
        upload_id = ''
    ):
        input_file = params[param_name]

        if input_file.source_type == 'path':
            size = os.stat(input_file.path).st_size
            input = open(input_file.path, 'rb')
        elif input_file.source_type == 'bytes':
            size = len(input_file.data)
            input = input_file.data

        if size < self._chunk_size:
            if input_file.source_type == 'path':
                input_file.data = input.read()

            params[param_name] = input_file
            return self.call(
                'post',
                path,
                headers,
                params
            )

        offset = 0
        counter = 0

        try:
            result = self.call('get', path + '/' + upload_id, headers)
            counter = result['chunksUploaded']
        except:
            pass

        if counter > 0:
            offset = counter * self._chunk_size
            input.seek(offset)

        while offset < size:
            if input_file.source_type == 'path':
                input_file.data = input.read(self._chunk_size) or input.read(size - offset)
            elif input_file.source_type == 'bytes':
                if offset + self._chunk_size < size:
                    end = offset + self._chunk_size
                else:
                    end = size
                input_file.data = input[offset:end]

            params[param_name] = input_file
            headers["content-range"] = f'bytes {offset}-{min((offset + self._chunk_size) - 1, size - 1)}/{size}'

            result = self.call(
                'post',
                path,
                headers,
                params,
            )

            offset = offset + self._chunk_size

            if "$id" in result:
                headers["x-appwrite-id"] = result["$id"]

            if on_progress is not None:
                end = min((((counter * self._chunk_size) + self._chunk_size) - 1), size - 1)
                on_progress({
                    "$id": result["$id"],
                    "progress": min(offset, size)/size * 100,
                    "sizeUploaded": end+1,
                    "chunksTotal": result["chunksTotal"],
                    "chunksUploaded": result["chunksUploaded"],
                })

            counter = counter + 1

        return result

    def flatten(self, data, prefix='', stringify=False):
        output = {}
        i = 0

        for key in data:
            value = data[key] if isinstance(data, dict) else key
            finalKey = prefix + '[' + key +']' if prefix else key
            finalKey = prefix + '[' + str(i) +']' if isinstance(data, list) else finalKey
            i += 1

            if isinstance(value, list) or isinstance(value, dict):
                output = {**output, **self.flatten(value, finalKey, stringify)}
            else:
                if stringify:
                    output[finalKey] = str(value)
                else:
                    output[finalKey] = value

        return output



from enum import Enum

class OAuthProvider(Enum):
    AMAZON = "amazon"
    APPLE = "apple"
    AUTH0 = "auth0"
    AUTHENTIK = "authentik"
    AUTODESK = "autodesk"
    BITBUCKET = "bitbucket"
    BITLY = "bitly"
    BOX = "box"
    DAILYMOTION = "dailymotion"
    DISCORD = "discord"
    DISQUS = "disqus"
    DROPBOX = "dropbox"
    ETSY = "etsy"
    FACEBOOK = "facebook"
    FIGMA = "figma"
    GITHUB = "github"
    GITLAB = "gitlab"
    GOOGLE = "google"
    LINKEDIN = "linkedin"
    MICROSOFT = "microsoft"
    NOTION = "notion"
    OIDC = "oidc"
    OKTA = "okta"
    PAYPAL = "paypal"
    PAYPALSANDBOX = "paypalSandbox"
    PODIO = "podio"
    SALESFORCE = "salesforce"
    SLACK = "slack"
    SPOTIFY = "spotify"
    STRIPE = "stripe"
    TRADESHIFT = "tradeshift"
    TRADESHIFTBOX = "tradeshiftBox"
    TWITCH = "twitch"
    WORDPRESS = "wordpress"
    YAHOO = "yahoo"
    YAMMER = "yammer"
    YANDEX = "yandex"
    ZOHO = "zoho"
    ZOOM = "zoom"
    MOCK = "mock"



import json


# Inherit from dict to allow for easy serialization
class Query():
    def __init__(self, method, attribute=None, values=None):
        self.method = method

        if attribute is not None:
            self.attribute = attribute

        if values is not None:
            self.values = values if isinstance(values, list) else [values]

    def __str__(self):
        return json.dumps(
            self.__dict__,
            separators=(",", ":"),
            default=lambda obj: obj.__dict__
        )

    @staticmethod
    def equal(attribute, value):
        return str(Query("equal", attribute, value))

    @staticmethod
    def not_equal(attribute, value):
        return str(Query("notEqual", attribute, value))

    @staticmethod
    def less_than(attribute, value):
        return str(Query("lessThan", attribute, value))

    @staticmethod
    def less_than_equal(attribute, value):
        return str(Query("lessThanEqual", attribute, value))

    @staticmethod
    def greater_than(attribute, value):
        return str(Query("greaterThan", attribute, value))

    @staticmethod
    def greater_than_equal(attribute, value):
        return str(Query("greaterThanEqual", attribute, value))

    @staticmethod
    def is_null(attribute):
        return str(Query("isNull", attribute, None))

    @staticmethod
    def is_not_null(attribute):
        return str(Query("isNotNull", attribute, None))

    @staticmethod
    def between(attribute, start, end):
        return str(Query("between", attribute, [start, end]))

    @staticmethod
    def starts_with(attribute, value):
        return str(Query("startsWith", attribute, value))

    @staticmethod
    def ends_with(attribute, value):
        return str(Query("endsWith", attribute, value))

    @staticmethod
    def select(attributes):
        return str(Query("select", None, attributes))

    @staticmethod
    def search(attribute, value):
        return str(Query("search", attribute, value))

    @staticmethod
    def order_asc(attribute):
        return str(Query("orderAsc", attribute, None))

    @staticmethod
    def order_desc(attribute):
        return str(Query("orderDesc", attribute, None))

    @staticmethod
    def cursor_before(id):
        return str(Query("cursorBefore", None, id))

    @staticmethod
    def cursor_after(id):
        return str(Query("cursorAfter", None, id))

    @staticmethod
    def limit(limit):
        return str(Query("limit", None, limit))

    @staticmethod
    def offset(offset):
        return str(Query("offset", None, offset))

    @staticmethod
    def contains(attribute, value):
        return str(Query("contains", attribute, value))

    @staticmethod
    def or_queries(queries):
        return str(Query("or", None, [json.loads(query) for query in queries]))

    @staticmethod
    def and_queries(queries):
        return str(Query("and", None, [json.loads(query) for query in queries]))



from ..service import Service
from typing import List, Dict, Any
from ..exception import RilzException
from ..enums.password_hash import PasswordHash;
from ..enums.authenticator_type import AuthenticatorType;
from ..enums.messaging_provider_type import MessagingProviderType;

class Users(Service):

    def __init__(self, client) -> None:
        super(Users, self).__init__(client)

    def list(self, queries: List[str] = None, search: str = None) -> Dict[str, Any]:
        """
        Get a list of all the project's users. You can use the query params to filter your results.

        Parameters
        ----------
        queries : List[str]
            Array of query strings generated using the Query class provided by the SDK. [Learn more about queries](https://rilck.com/docs/queries). Maximum of 100 queries are allowed, each 4096 characters long. You may filter on the following attributes: name, email, phone, status, passwordUpdate, registration, emailVerification, phoneVerification, labels
        search : str
            Search term to filter your list results. Max length: 256 chars.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/users'
        api_params = {}

        api_params['queries'] = queries
        api_params['search'] = search

        return self.client.call('get', api_path, {
        }, api_params)

    def create(self, user_id: str, email: str = None, phone: str = None, password: str = None, name: str = None) -> Dict[str, Any]:
        """
        Create a new user.

        Parameters
        ----------
        user_id : str
            User ID. Choose a custom ID or generate a random ID with `ID.unique()`. Valid chars are a-z, A-Z, 0-9, period, hyphen, and underscore. Can't start with a special char. Max length is 36 chars.
        email : str
            User email.
        phone : str
            Phone number. Format this number with a leading '+' and a country code, e.g., +16175551212.
        password : str
            Plain text user password. Must be at least 8 chars.
        name : str
            User name. Max length: 128 chars.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/users'
        api_params = {}
        if user_id is None:
            raise RilzException('Missing required parameter: "user_id"')


        api_params['userId'] = user_id
        api_params['email'] = email
        api_params['phone'] = phone
        api_params['password'] = password
        api_params['name'] = name

        return self.client.call('post', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def create_argon2_user(self, user_id: str, email: str, password: str, name: str = None) -> Dict[str, Any]:
        """
        Create a new user. Password provided must be hashed with the [Argon2](https://en.wikipedia.org/wiki/Argon2) algorithm. Use the [POST /users](https://rilck.com/docs/server/users#usersCreate) endpoint to create users with a plain text password.

        Parameters
        ----------
        user_id : str
            User ID. Choose a custom ID or generate a random ID with `ID.unique()`. Valid chars are a-z, A-Z, 0-9, period, hyphen, and underscore. Can't start with a special char. Max length is 36 chars.
        email : str
            User email.
        password : str
            User password hashed using Argon2.
        name : str
            User name. Max length: 128 chars.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/users/argon2'
        api_params = {}
        if user_id is None:
            raise RilzException('Missing required parameter: "user_id"')

        if email is None:
            raise RilzException('Missing required parameter: "email"')

        if password is None:
            raise RilzException('Missing required parameter: "password"')


        api_params['userId'] = user_id
        api_params['email'] = email
        api_params['password'] = password
        api_params['name'] = name

        return self.client.call('post', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def create_bcrypt_user(self, user_id: str, email: str, password: str, name: str = None) -> Dict[str, Any]:
        """
        Create a new user. Password provided must be hashed with the [Bcrypt](https://en.wikipedia.org/wiki/Bcrypt) algorithm. Use the [POST /users](https://rilck.com/docs/server/users#usersCreate) endpoint to create users with a plain text password.

        Parameters
        ----------
        user_id : str
            User ID. Choose a custom ID or generate a random ID with `ID.unique()`. Valid chars are a-z, A-Z, 0-9, period, hyphen, and underscore. Can't start with a special char. Max length is 36 chars.
        email : str
            User email.
        password : str
            User password hashed using Bcrypt.
        name : str
            User name. Max length: 128 chars.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/users/bcrypt'
        api_params = {}
        if user_id is None:
            raise RilzException('Missing required parameter: "user_id"')

        if email is None:
            raise RilzException('Missing required parameter: "email"')

        if password is None:
            raise RilzException('Missing required parameter: "password"')


        api_params['userId'] = user_id
        api_params['email'] = email
        api_params['password'] = password
        api_params['name'] = name

        return self.client.call('post', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def list_identities(self, queries: List[str] = None, search: str = None) -> Dict[str, Any]:
        """
        Get identities for all users.

        Parameters
        ----------
        queries : List[str]
            Array of query strings generated using the Query class provided by the SDK. [Learn more about queries](https://rilck.com/docs/queries). Maximum of 100 queries are allowed, each 4096 characters long. You may filter on the following attributes: userId, provider, providerUid, providerEmail, providerAccessTokenExpiry
        search : str
            Search term to filter your list results. Max length: 256 chars.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/users/identities'
        api_params = {}

        api_params['queries'] = queries
        api_params['search'] = search

        return self.client.call('get', api_path, {
        }, api_params)

    def delete_identity(self, identity_id: str) -> Dict[str, Any]:
        """
        Delete an identity by its unique ID.

        Parameters
        ----------
        identity_id : str
            Identity ID.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/users/identities/{identityId}'
        api_params = {}
        if identity_id is None:
            raise RilzException('Missing required parameter: "identity_id"')

        api_path = api_path.replace('{identityId}', identity_id)


        return self.client.call('delete', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def create_md5_user(self, user_id: str, email: str, password: str, name: str = None) -> Dict[str, Any]:
        """
        Create a new user. Password provided must be hashed with the [MD5](https://en.wikipedia.org/wiki/MD5) algorithm. Use the [POST /users](https://rilck.com/docs/server/users#usersCreate) endpoint to create users with a plain text password.

        Parameters
        ----------
        user_id : str
            User ID. Choose a custom ID or generate a random ID with `ID.unique()`. Valid chars are a-z, A-Z, 0-9, period, hyphen, and underscore. Can't start with a special char. Max length is 36 chars.
        email : str
            User email.
        password : str
            User password hashed using MD5.
        name : str
            User name. Max length: 128 chars.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/users/md5'
        api_params = {}
        if user_id is None:
            raise RilzException('Missing required parameter: "user_id"')

        if email is None:
            raise RilzException('Missing required parameter: "email"')

        if password is None:
            raise RilzException('Missing required parameter: "password"')


        api_params['userId'] = user_id
        api_params['email'] = email
        api_params['password'] = password
        api_params['name'] = name

        return self.client.call('post', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def create_ph_pass_user(self, user_id: str, email: str, password: str, name: str = None) -> Dict[str, Any]:
        """
        Create a new user. Password provided must be hashed with the [PHPass](https://www.openwall.com/phpass/) algorithm. Use the [POST /users](https://rilck.com/docs/server/users#usersCreate) endpoint to create users with a plain text password.

        Parameters
        ----------
        user_id : str
            User ID. Choose a custom ID or pass the string `ID.unique()`to auto generate it. Valid chars are a-z, A-Z, 0-9, period, hyphen, and underscore. Can't start with a special char. Max length is 36 chars.
        email : str
            User email.
        password : str
            User password hashed using PHPass.
        name : str
            User name. Max length: 128 chars.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/users/phpass'
        api_params = {}
        if user_id is None:
            raise RilzException('Missing required parameter: "user_id"')

        if email is None:
            raise RilzException('Missing required parameter: "email"')

        if password is None:
            raise RilzException('Missing required parameter: "password"')


        api_params['userId'] = user_id
        api_params['email'] = email
        api_params['password'] = password
        api_params['name'] = name

        return self.client.call('post', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def create_scrypt_user(self, user_id: str, email: str, password: str, password_salt: str, password_cpu: float, password_memory: float, password_parallel: float, password_length: float, name: str = None) -> Dict[str, Any]:
        """
        Create a new user. Password provided must be hashed with the [Scrypt](https://github.com/Tarsnap/scrypt) algorithm. Use the [POST /users](https://rilck.com/docs/server/users#usersCreate) endpoint to create users with a plain text password.

        Parameters
        ----------
        user_id : str
            User ID. Choose a custom ID or generate a random ID with `ID.unique()`. Valid chars are a-z, A-Z, 0-9, period, hyphen, and underscore. Can't start with a special char. Max length is 36 chars.
        email : str
            User email.
        password : str
            User password hashed using Scrypt.
        password_salt : str
            Optional salt used to hash password.
        password_cpu : float
            Optional CPU cost used to hash password.
        password_memory : float
            Optional memory cost used to hash password.
        password_parallel : float
            Optional parallelization cost used to hash password.
        password_length : float
            Optional hash length used to hash password.
        name : str
            User name. Max length: 128 chars.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/users/scrypt'
        api_params = {}
        if user_id is None:
            raise RilzException('Missing required parameter: "user_id"')

        if email is None:
            raise RilzException('Missing required parameter: "email"')

        if password is None:
            raise RilzException('Missing required parameter: "password"')

        if password_salt is None:
            raise RilzException('Missing required parameter: "password_salt"')

        if password_cpu is None:
            raise RilzException('Missing required parameter: "password_cpu"')

        if password_memory is None:
            raise RilzException('Missing required parameter: "password_memory"')

        if password_parallel is None:
            raise RilzException('Missing required parameter: "password_parallel"')

        if password_length is None:
            raise RilzException('Missing required parameter: "password_length"')


        api_params['userId'] = user_id
        api_params['email'] = email
        api_params['password'] = password
        api_params['passwordSalt'] = password_salt
        api_params['passwordCpu'] = password_cpu
        api_params['passwordMemory'] = password_memory
        api_params['passwordParallel'] = password_parallel
        api_params['passwordLength'] = password_length
        api_params['name'] = name

        return self.client.call('post', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def create_scrypt_modified_user(self, user_id: str, email: str, password: str, password_salt: str, password_salt_separator: str, password_signer_key: str, name: str = None) -> Dict[str, Any]:
        """
        Create a new user. Password provided must be hashed with the [Scrypt Modified](https://gist.github.com/Meldiron/eecf84a0225eccb5a378d45bb27462cc) algorithm. Use the [POST /users](https://rilck.com/docs/server/users#usersCreate) endpoint to create users with a plain text password.

        Parameters
        ----------
        user_id : str
            User ID. Choose a custom ID or generate a random ID with `ID.unique()`. Valid chars are a-z, A-Z, 0-9, period, hyphen, and underscore. Can't start with a special char. Max length is 36 chars.
        email : str
            User email.
        password : str
            User password hashed using Scrypt Modified.
        password_salt : str
            Salt used to hash password.
        password_salt_separator : str
            Salt separator used to hash password.
        password_signer_key : str
            Signer key used to hash password.
        name : str
            User name. Max length: 128 chars.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/users/scrypt-modified'
        api_params = {}
        if user_id is None:
            raise RilzException('Missing required parameter: "user_id"')

        if email is None:
            raise RilzException('Missing required parameter: "email"')

        if password is None:
            raise RilzException('Missing required parameter: "password"')

        if password_salt is None:
            raise RilzException('Missing required parameter: "password_salt"')

        if password_salt_separator is None:
            raise RilzException('Missing required parameter: "password_salt_separator"')

        if password_signer_key is None:
            raise RilzException('Missing required parameter: "password_signer_key"')


        api_params['userId'] = user_id
        api_params['email'] = email
        api_params['password'] = password
        api_params['passwordSalt'] = password_salt
        api_params['passwordSaltSeparator'] = password_salt_separator
        api_params['passwordSignerKey'] = password_signer_key
        api_params['name'] = name

        return self.client.call('post', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def create_sha_user(self, user_id: str, email: str, password: str, password_version: PasswordHash = None, name: str = None) -> Dict[str, Any]:
        """
        Create a new user. Password provided must be hashed with the [SHA](https://en.wikipedia.org/wiki/Secure_Hash_Algorithm) algorithm. Use the [POST /users](https://rilck.com/docs/server/users#usersCreate) endpoint to create users with a plain text password.

        Parameters
        ----------
        user_id : str
            User ID. Choose a custom ID or generate a random ID with `ID.unique()`. Valid chars are a-z, A-Z, 0-9, period, hyphen, and underscore. Can't start with a special char. Max length is 36 chars.
        email : str
            User email.
        password : str
            User password hashed using SHA.
        password_version : PasswordHash
            Optional SHA version used to hash password. Allowed values are: 'sha1', 'sha224', 'sha256', 'sha384', 'sha512/224', 'sha512/256', 'sha512', 'sha3-224', 'sha3-256', 'sha3-384', 'sha3-512'
        name : str
            User name. Max length: 128 chars.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/users/sha'
        api_params = {}
        if user_id is None:
            raise RilzException('Missing required parameter: "user_id"')

        if email is None:
            raise RilzException('Missing required parameter: "email"')

        if password is None:
            raise RilzException('Missing required parameter: "password"')


        api_params['userId'] = user_id
        api_params['email'] = email
        api_params['password'] = password
        api_params['passwordVersion'] = password_version
        api_params['name'] = name

        return self.client.call('post', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def get(self, user_id: str) -> Dict[str, Any]:
        """
        Get a user by its unique ID.

        Parameters
        ----------
        user_id : str
            User ID.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/users/{userId}'
        api_params = {}
        if user_id is None:
            raise RilzException('Missing required parameter: "user_id"')

        api_path = api_path.replace('{userId}', user_id)


        return self.client.call('get', api_path, {
        }, api_params)

    def delete(self, user_id: str) -> Dict[str, Any]:
        """
        Delete a user by its unique ID, thereby releasing it's ID. Since ID is released and can be reused, all user-related resources like documents or storage files should be deleted before user deletion. If you want to keep ID reserved, use the [updateStatus](https://rilck.com/docs/server/users#usersUpdateStatus) endpoint instead.

        Parameters
        ----------
        user_id : str
            User ID.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/users/{userId}'
        api_params = {}
        if user_id is None:
            raise RilzException('Missing required parameter: "user_id"')

        api_path = api_path.replace('{userId}', user_id)


        return self.client.call('delete', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def update_email(self, user_id: str, email: str) -> Dict[str, Any]:
        """
        Update the user email by its unique ID.

        Parameters
        ----------
        user_id : str
            User ID.
        email : str
            User email.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/users/{userId}/email'
        api_params = {}
        if user_id is None:
            raise RilzException('Missing required parameter: "user_id"')

        if email is None:
            raise RilzException('Missing required parameter: "email"')

        api_path = api_path.replace('{userId}', user_id)

        api_params['email'] = email

        return self.client.call('patch', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def create_jwt(self, user_id: str, session_id: str = None, duration: float = None) -> Dict[str, Any]:
        """
        Use this endpoint to create a JSON Web Token for user by its unique ID. You can use the resulting JWT to authenticate on behalf of the user. The JWT secret will become invalid if the session it uses gets deleted.

        Parameters
        ----------
        user_id : str
            User ID.
        session_id : str
            Session ID. Use the string 'recent' to use the most recent session. Defaults to the most recent session.
        duration : float
            Time in seconds before JWT expires. Default duration is 900 seconds, and maximum is 3600 seconds.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/users/{userId}/jwts'
        api_params = {}
        if user_id is None:
            raise RilzException('Missing required parameter: "user_id"')

        api_path = api_path.replace('{userId}', user_id)

        api_params['sessionId'] = session_id
        api_params['duration'] = duration

        return self.client.call('post', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def update_labels(self, user_id: str, labels: List[str]) -> Dict[str, Any]:
        """
        Update the user labels by its unique ID. 
        
        Labels can be used to grant access to resources. While teams are a way for user's to share access to a resource, labels can be defined by the developer to grant access without an invitation. See the [Permissions docs](https://rilck.com/docs/permissions) for more info.

        Parameters
        ----------
        user_id : str
            User ID.
        labels : List[str]
            Array of user labels. Replaces the previous labels. Maximum of 1000 labels are allowed, each up to 36 alphanumeric characters long.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/users/{userId}/labels'
        api_params = {}
        if user_id is None:
            raise RilzException('Missing required parameter: "user_id"')

        if labels is None:
            raise RilzException('Missing required parameter: "labels"')

        api_path = api_path.replace('{userId}', user_id)

        api_params['labels'] = labels

        return self.client.call('put', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def list_logs(self, user_id: str, queries: List[str] = None) -> Dict[str, Any]:
        """
        Get the user activity logs list by its unique ID.

        Parameters
        ----------
        user_id : str
            User ID.
        queries : List[str]
            Array of query strings generated using the Query class provided by the SDK. [Learn more about queries](https://rilck.com/docs/queries). Only supported methods are limit and offset
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/users/{userId}/logs'
        api_params = {}
        if user_id is None:
            raise RilzException('Missing required parameter: "user_id"')

        api_path = api_path.replace('{userId}', user_id)

        api_params['queries'] = queries

        return self.client.call('get', api_path, {
        }, api_params)

    def list_memberships(self, user_id: str, queries: List[str] = None, search: str = None) -> Dict[str, Any]:
        """
        Get the user membership list by its unique ID.

        Parameters
        ----------
        user_id : str
            User ID.
        queries : List[str]
            Array of query strings generated using the Query class provided by the SDK. [Learn more about queries](https://rilck.com/docs/queries). Maximum of 100 queries are allowed, each 4096 characters long. You may filter on the following attributes: userId, teamId, invited, joined, confirm, roles
        search : str
            Search term to filter your list results. Max length: 256 chars.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/users/{userId}/memberships'
        api_params = {}
        if user_id is None:
            raise RilzException('Missing required parameter: "user_id"')

        api_path = api_path.replace('{userId}', user_id)

        api_params['queries'] = queries
        api_params['search'] = search

        return self.client.call('get', api_path, {
        }, api_params)

    def update_mfa(self, user_id: str, mfa: bool) -> Dict[str, Any]:
        """
        Enable or disable MFA on a user account.

        Parameters
        ----------
        user_id : str
            User ID.
        mfa : bool
            Enable or disable MFA.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/users/{userId}/mfa'
        api_params = {}
        if user_id is None:
            raise RilzException('Missing required parameter: "user_id"')

        if mfa is None:
            raise RilzException('Missing required parameter: "mfa"')

        api_path = api_path.replace('{userId}', user_id)

        api_params['mfa'] = mfa

        return self.client.call('patch', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def delete_mfa_authenticator(self, user_id: str, type: AuthenticatorType) -> Dict[str, Any]:
        """
        Delete an authenticator app.

        Parameters
        ----------
        user_id : str
            User ID.
        type : AuthenticatorType
            Type of authenticator.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/users/{userId}/mfa/authenticators/{type}'
        api_params = {}
        if user_id is None:
            raise RilzException('Missing required parameter: "user_id"')

        if type is None:
            raise RilzException('Missing required parameter: "type"')

        api_path = api_path.replace('{userId}', user_id)
        api_path = api_path.replace('{type}', type)


        return self.client.call('delete', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def list_mfa_factors(self, user_id: str) -> Dict[str, Any]:
        """
        List the factors available on the account to be used as a MFA challange.

        Parameters
        ----------
        user_id : str
            User ID.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/users/{userId}/mfa/factors'
        api_params = {}
        if user_id is None:
            raise RilzException('Missing required parameter: "user_id"')

        api_path = api_path.replace('{userId}', user_id)


        return self.client.call('get', api_path, {
        }, api_params)

    def get_mfa_recovery_codes(self, user_id: str) -> Dict[str, Any]:
        """
        Get recovery codes that can be used as backup for MFA flow by User ID. Before getting codes, they must be generated using [createMfaRecoveryCodes](/docs/references/cloud/client-web/account#createMfaRecoveryCodes) method.

        Parameters
        ----------
        user_id : str
            User ID.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/users/{userId}/mfa/recovery-codes'
        api_params = {}
        if user_id is None:
            raise RilzException('Missing required parameter: "user_id"')

        api_path = api_path.replace('{userId}', user_id)


        return self.client.call('get', api_path, {
        }, api_params)

    def update_mfa_recovery_codes(self, user_id: str) -> Dict[str, Any]:
        """
        Regenerate recovery codes that can be used as backup for MFA flow by User ID. Before regenerating codes, they must be first generated using [createMfaRecoveryCodes](/docs/references/cloud/client-web/account#createMfaRecoveryCodes) method.

        Parameters
        ----------
        user_id : str
            User ID.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/users/{userId}/mfa/recovery-codes'
        api_params = {}
        if user_id is None:
            raise RilzException('Missing required parameter: "user_id"')

        api_path = api_path.replace('{userId}', user_id)


        return self.client.call('put', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def create_mfa_recovery_codes(self, user_id: str) -> Dict[str, Any]:
        """
        Generate recovery codes used as backup for MFA flow for User ID. Recovery codes can be used as a MFA verification type in [createMfaChallenge](/docs/references/cloud/client-web/account#createMfaChallenge) method by client SDK.

        Parameters
        ----------
        user_id : str
            User ID.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/users/{userId}/mfa/recovery-codes'
        api_params = {}
        if user_id is None:
            raise RilzException('Missing required parameter: "user_id"')

        api_path = api_path.replace('{userId}', user_id)


        return self.client.call('patch', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def update_name(self, user_id: str, name: str) -> Dict[str, Any]:
        """
        Update the user name by its unique ID.

        Parameters
        ----------
        user_id : str
            User ID.
        name : str
            User name. Max length: 128 chars.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/users/{userId}/name'
        api_params = {}
        if user_id is None:
            raise RilzException('Missing required parameter: "user_id"')

        if name is None:
            raise RilzException('Missing required parameter: "name"')

        api_path = api_path.replace('{userId}', user_id)

        api_params['name'] = name

        return self.client.call('patch', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def update_password(self, user_id: str, password: str) -> Dict[str, Any]:
        """
        Update the user password by its unique ID.

        Parameters
        ----------
        user_id : str
            User ID.
        password : str
            New user password. Must be at least 8 chars.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/users/{userId}/password'
        api_params = {}
        if user_id is None:
            raise RilzException('Missing required parameter: "user_id"')

        if password is None:
            raise RilzException('Missing required parameter: "password"')

        api_path = api_path.replace('{userId}', user_id)

        api_params['password'] = password

        return self.client.call('patch', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def update_phone(self, user_id: str, number: str) -> Dict[str, Any]:
        """
        Update the user phone by its unique ID.

        Parameters
        ----------
        user_id : str
            User ID.
        number : str
            User phone number.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/users/{userId}/phone'
        api_params = {}
        if user_id is None:
            raise RilzException('Missing required parameter: "user_id"')

        if number is None:
            raise RilzException('Missing required parameter: "number"')

        api_path = api_path.replace('{userId}', user_id)

        api_params['number'] = number

        return self.client.call('patch', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def get_prefs(self, user_id: str) -> Dict[str, Any]:
        """
        Get the user preferences by its unique ID.

        Parameters
        ----------
        user_id : str
            User ID.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/users/{userId}/prefs'
        api_params = {}
        if user_id is None:
            raise RilzException('Missing required parameter: "user_id"')

        api_path = api_path.replace('{userId}', user_id)


        return self.client.call('get', api_path, {
        }, api_params)

    def update_prefs(self, user_id: str, prefs: dict) -> Dict[str, Any]:
        """
        Update the user preferences by its unique ID. The object you pass is stored as is, and replaces any previous value. The maximum allowed prefs size is 64kB and throws error if exceeded.

        Parameters
        ----------
        user_id : str
            User ID.
        prefs : dict
            Prefs key-value JSON object.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/users/{userId}/prefs'
        api_params = {}
        if user_id is None:
            raise RilzException('Missing required parameter: "user_id"')

        if prefs is None:
            raise RilzException('Missing required parameter: "prefs"')

        api_path = api_path.replace('{userId}', user_id)

        api_params['prefs'] = prefs

        return self.client.call('patch', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def list_sessions(self, user_id: str) -> Dict[str, Any]:
        """
        Get the user sessions list by its unique ID.

        Parameters
        ----------
        user_id : str
            User ID.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/users/{userId}/sessions'
        api_params = {}
        if user_id is None:
            raise RilzException('Missing required parameter: "user_id"')

        api_path = api_path.replace('{userId}', user_id)


        return self.client.call('get', api_path, {
        }, api_params)

    def create_session(self, user_id: str) -> Dict[str, Any]:
        """
        Creates a session for a user. Returns an immediately usable session object.
        
        If you want to generate a token for a custom authentication flow, use the [POST /users/{userId}/tokens](https://rilck.com/docs/server/users#createToken) endpoint.

        Parameters
        ----------
        user_id : str
            User ID. Choose a custom ID or generate a random ID with `ID.unique()`. Valid chars are a-z, A-Z, 0-9, period, hyphen, and underscore. Can't start with a special char. Max length is 36 chars.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/users/{userId}/sessions'
        api_params = {}
        if user_id is None:
            raise RilzException('Missing required parameter: "user_id"')

        api_path = api_path.replace('{userId}', user_id)


        return self.client.call('post', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def delete_sessions(self, user_id: str) -> Dict[str, Any]:
        """
        Delete all user's sessions by using the user's unique ID.

        Parameters
        ----------
        user_id : str
            User ID.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/users/{userId}/sessions'
        api_params = {}
        if user_id is None:
            raise RilzException('Missing required parameter: "user_id"')

        api_path = api_path.replace('{userId}', user_id)


        return self.client.call('delete', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def delete_session(self, user_id: str, session_id: str) -> Dict[str, Any]:
        """
        Delete a user sessions by its unique ID.

        Parameters
        ----------
        user_id : str
            User ID.
        session_id : str
            Session ID.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/users/{userId}/sessions/{sessionId}'
        api_params = {}
        if user_id is None:
            raise RilzException('Missing required parameter: "user_id"')

        if session_id is None:
            raise RilzException('Missing required parameter: "session_id"')

        api_path = api_path.replace('{userId}', user_id)
        api_path = api_path.replace('{sessionId}', session_id)


        return self.client.call('delete', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def update_status(self, user_id: str, status: bool) -> Dict[str, Any]:
        """
        Update the user status by its unique ID. Use this endpoint as an alternative to deleting a user if you want to keep user's ID reserved.

        Parameters
        ----------
        user_id : str
            User ID.
        status : bool
            User Status. To activate the user pass `true` and to block the user pass `false`.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/users/{userId}/status'
        api_params = {}
        if user_id is None:
            raise RilzException('Missing required parameter: "user_id"')

        if status is None:
            raise RilzException('Missing required parameter: "status"')

        api_path = api_path.replace('{userId}', user_id)

        api_params['status'] = status

        return self.client.call('patch', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def list_targets(self, user_id: str, queries: List[str] = None) -> Dict[str, Any]:
        """
        List the messaging targets that are associated with a user.

        Parameters
        ----------
        user_id : str
            User ID.
        queries : List[str]
            Array of query strings generated using the Query class provided by the SDK. [Learn more about queries](https://rilck.com/docs/queries). Maximum of 100 queries are allowed, each 4096 characters long. You may filter on the following attributes: name, email, phone, status, passwordUpdate, registration, emailVerification, phoneVerification, labels
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/users/{userId}/targets'
        api_params = {}
        if user_id is None:
            raise RilzException('Missing required parameter: "user_id"')

        api_path = api_path.replace('{userId}', user_id)

        api_params['queries'] = queries

        return self.client.call('get', api_path, {
        }, api_params)

    def create_target(self, user_id: str, target_id: str, provider_type: MessagingProviderType, identifier: str, provider_id: str = None, name: str = None) -> Dict[str, Any]:
        """
        Create a messaging target.

        Parameters
        ----------
        user_id : str
            User ID.
        target_id : str
            Target ID. Choose a custom ID or generate a random ID with `ID.unique()`. Valid chars are a-z, A-Z, 0-9, period, hyphen, and underscore. Can't start with a special char. Max length is 36 chars.
        provider_type : MessagingProviderType
            The target provider type. Can be one of the following: `email`, `sms` or `push`.
        identifier : str
            The target identifier (token, email, phone etc.)
        provider_id : str
            Provider ID. Message will be sent to this target from the specified provider ID. If no provider ID is set the first setup provider will be used.
        name : str
            Target name. Max length: 128 chars. For example: My Awesome App Galaxy S23.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/users/{userId}/targets'
        api_params = {}
        if user_id is None:
            raise RilzException('Missing required parameter: "user_id"')

        if target_id is None:
            raise RilzException('Missing required parameter: "target_id"')

        if provider_type is None:
            raise RilzException('Missing required parameter: "provider_type"')

        if identifier is None:
            raise RilzException('Missing required parameter: "identifier"')

        api_path = api_path.replace('{userId}', user_id)

        api_params['targetId'] = target_id
        api_params['providerType'] = provider_type
        api_params['identifier'] = identifier
        api_params['providerId'] = provider_id
        api_params['name'] = name

        return self.client.call('post', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def get_target(self, user_id: str, target_id: str) -> Dict[str, Any]:
        """
        Get a user's push notification target by ID.

        Parameters
        ----------
        user_id : str
            User ID.
        target_id : str
            Target ID.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/users/{userId}/targets/{targetId}'
        api_params = {}
        if user_id is None:
            raise RilzException('Missing required parameter: "user_id"')

        if target_id is None:
            raise RilzException('Missing required parameter: "target_id"')

        api_path = api_path.replace('{userId}', user_id)
        api_path = api_path.replace('{targetId}', target_id)


        return self.client.call('get', api_path, {
        }, api_params)

    def update_target(self, user_id: str, target_id: str, identifier: str = None, provider_id: str = None, name: str = None) -> Dict[str, Any]:
        """
        Update a messaging target.

        Parameters
        ----------
        user_id : str
            User ID.
        target_id : str
            Target ID.
        identifier : str
            The target identifier (token, email, phone etc.)
        provider_id : str
            Provider ID. Message will be sent to this target from the specified provider ID. If no provider ID is set the first setup provider will be used.
        name : str
            Target name. Max length: 128 chars. For example: My Awesome App Galaxy S23.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/users/{userId}/targets/{targetId}'
        api_params = {}
        if user_id is None:
            raise RilzException('Missing required parameter: "user_id"')

        if target_id is None:
            raise RilzException('Missing required parameter: "target_id"')

        api_path = api_path.replace('{userId}', user_id)
        api_path = api_path.replace('{targetId}', target_id)

        api_params['identifier'] = identifier
        api_params['providerId'] = provider_id
        api_params['name'] = name

        return self.client.call('patch', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def delete_target(self, user_id: str, target_id: str) -> Dict[str, Any]:
        """
        Delete a messaging target.

        Parameters
        ----------
        user_id : str
            User ID.
        target_id : str
            Target ID.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/users/{userId}/targets/{targetId}'
        api_params = {}
        if user_id is None:
            raise RilzException('Missing required parameter: "user_id"')

        if target_id is None:
            raise RilzException('Missing required parameter: "target_id"')

        api_path = api_path.replace('{userId}', user_id)
        api_path = api_path.replace('{targetId}', target_id)


        return self.client.call('delete', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def create_token(self, user_id: str, length: float = None, expire: float = None) -> Dict[str, Any]:
        """
        Returns a token with a secret key for creating a session. Use the user ID and secret and submit a request to the [PUT /account/sessions/token](https://rilck.com/docs/references/cloud/client-web/account#createSession) endpoint to complete the login process.
        

        Parameters
        ----------
        user_id : str
            User ID.
        length : float
            Token length in characters. The default length is 6 characters
        expire : float
            Token expiration period in seconds. The default expiration is 15 minutes.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/users/{userId}/tokens'
        api_params = {}
        if user_id is None:
            raise RilzException('Missing required parameter: "user_id"')

        api_path = api_path.replace('{userId}', user_id)

        api_params['length'] = length
        api_params['expire'] = expire

        return self.client.call('post', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def update_email_verification(self, user_id: str, email_verification: bool) -> Dict[str, Any]:
        """
        Update the user email verification status by its unique ID.

        Parameters
        ----------
        user_id : str
            User ID.
        email_verification : bool
            User email verification status.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/users/{userId}/verification'
        api_params = {}
        if user_id is None:
            raise RilzException('Missing required parameter: "user_id"')

        if email_verification is None:
            raise RilzException('Missing required parameter: "email_verification"')

        api_path = api_path.replace('{userId}', user_id)

        api_params['emailVerification'] = email_verification

        return self.client.call('patch', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def update_phone_verification(self, user_id: str, phone_verification: bool) -> Dict[str, Any]:
        """
        Update the user phone verification status by its unique ID.

        Parameters
        ----------
        user_id : str
            User ID.
        phone_verification : bool
            User phone verification status.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/users/{userId}/verification/phone'
        api_params = {}
        if user_id is None:
            raise RilzException('Missing required parameter: "user_id"')

        if phone_verification is None:
            raise RilzException('Missing required parameter: "phone_verification"')

        api_path = api_path.replace('{userId}', user_id)

        api_params['phoneVerification'] = phone_verification

        return self.client.call('patch', api_path, {
            'content-type': 'application/json',
        }, api_params)



from selenium import webdriver
from selenium.webdriver.common.by import By
from selenium.webdriver.support.wait import WebDriverWait
from selenium.webdriver.support.select import Select
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.chrome.options import Options
from selenium.webdriver.common.action_chains import ActionChains
from time import sleep
from selenium.webdriver.remote.webelement import WebElement
from webdriver_manager.chrome import ChromeDriverManager
from selenium.webdriver.chrome.service import Service
from datetime import datetime
from selenium.webdriver.common.keys import Keys
# exceptions
from selenium.common.exceptions import (
    NoSuchElementException,
    StaleElementReferenceException,
    ElementClickInterceptedException,
    TimeoutException,
    InvalidElementStateException,
)

from rilz.logger import logger

from rilz.browser.exceptions import WindowNotFound

import os
from contextlib import contextmanager
from typing import Iterator, List, Callable, Any
import pathlib
import uuid
import shutil

class _Browser:
    driver: webdriver.Chrome | webdriver.Remote
    driver_wait: WebDriverWait
    actions: ActionChains

    @contextmanager
    def on_new_window(self, url: str) -> Iterator[None]:
        """Open a new window with the given url."""
        last_handle = self.driver.current_window_handle
        self.driver.execute_script(f"window.open('{url}')")
        new_handle = None
        while not new_handle:
            for handle in self.driver.window_handles:
                if handle is not last_handle:
                    # check if it has the url
                    self.driver.switch_to.window(handle)
                    if self.driver.current_url == url:
                        if self.driver.execute_script("return document.readyState") == "complete":
                            new_handle = handle
                            break
            sleep(1)
        yield
        self.driver.close()
        self.driver.switch_to.window(last_handle)

    @contextmanager
    def on_window(self, has_element: str, retry: int = 10):
        """Switch to a window that has the given element."""
        default_handle = self.driver.current_window_handle
        found_handle = None
        while not found_handle:
            try:
                for handle in self.driver.window_handles:
                    if handle is default_handle:
                        continue
                    self.driver.switch_to.window(handle)
                    if self.driver.find_elements(By.XPATH, has_element):
                        found_handle = handle
                        break
            except:
                sleep(1)
                retry -= 1
                if retry < 1:
                    raise WindowNotFound(f"Window with xpath {has_element} not found")
        yield
        self.driver.switch_to.window(default_handle)

    @contextmanager
    def on_iframe(self, xpath: str) -> Iterator[None]:
        """Switch to an iframe."""
        iframe = self.driver_wait.until(EC.presence_of_element_located((By.XPATH, xpath)))
        self.driver.switch_to.frame(iframe)
        try:
            yield
        finally:
            self.driver.switch_to.default_content()

    def wait_alert(self, retry: int = 1) -> Any:
        """Wait for an alert to appear."""
        while retry > 0:
            try:
                return self.driver_wait.until(EC.alert_is_present())
            except TimeoutException:
                retry -= 1
        raise TimeoutException("Alert not found")

    def __get_condition(self, condition: str) -> Callable:
        match condition:
            case "visible":
                return EC.visibility_of_element_located
            case "visible_any":
                return EC.visibility_of_any_elements_located
            case "visible_all":
                return EC.visibility_of_all_elements_located
            case "clickable":
                return EC.element_to_be_clickable
            case "selected":
                return EC.element_to_be_selected
            case "located_all":
                return EC.presence_of_all_elements_located
            case _:
                return EC.presence_of_element_located

    def __get_element(self, xpath: str, ec: str, retry: int = 3):
        """Wait until the element is located."""
        while True:
            try:
                condition_func = self.__get_condition(ec)
                return self.driver_wait.until(condition_func((By.XPATH, xpath)))
            except (TimeoutException, NoSuchElementException, StaleElementReferenceException):
                retry -= 1
                if retry < 1:
                    raise NoSuchElementException(f"Element with xpath {xpath} not found")

    def find_elements(self, xpath: str, condition: str = "located_all", wait: bool = True, retry: int = 3) -> List[WebElement]:
        """Wait until the any of the elements are located."""
        if wait:
            try:
                self.__get_element(xpath, ec=condition, retry=retry)
            except NoSuchElementException:
                return []
        return self.driver.find_elements(By.XPATH, xpath)

    def find_element(self, xpath: str, condition: str = None, retry: int = 1) -> List[WebElement] | WebElement:
        """Wait until the element is located."""
        return self.__get_element(xpath, ec=condition, retry=retry)

    def scroll_to(self, xpath: str, condition: str = "visible", actions: bool = False) -> None:
        """Scroll to the element."""
        element = self.find_element(xpath, condition)
        if actions:
            self.actions.reset_actions()
            self.actions.move_to_element(element).perform()
        else:
            self.driver.execute_script(
                "arguments[0].scrollIntoView(true);"
                "arguments[0].style.border='3px solid red';",
                element,
            )

    def hover(self, xpath: str, condition: str = "visible", dom=True) -> None:
        """Hover over the element."""
        element = self.find_element(xpath, condition)
        if not dom:
            self.actions.reset_actions()
            self.actions.move_to_element(element).perform()
        else:
            self.driver.execute_script(
                "arguments[0].scrollIntoView(true);"
                "arguments[0].style.border='3px solid red';"
                "if (arguments[0].onmouseover) {arguments[0].onmouseover();}",
                element,
            )

    def click(self, xpath: str, max_retries: int = 3, actions:bool=False, double:bool=False, condition='clickable') -> None:
        """Click the element."""
        retries = 0
        stale_retry = 0
        while retries < max_retries:
            try:
                element = self.find_element(xpath, condition=condition)        
                if actions:
                    self.actions.reset_actions()
                    if double:
                        self.actions.key_down(Keys.CONTROL).click(element).pause(0.5).key_up(Keys.CONTROL).perform()
                    else:
                        self.actions.click(element).perform()
                    return
                try:
                    self.driver.execute_script(
                        "if (arguments[0].onmouseover) {arguments[0].onmouseover();}",
                        "if (arguments[0].onclick) {arguments[0].onclick();} else {arguments[0].click();}",
                        "arguments[0].scrollIntoView(true);",
                        element,
                    )
                    element.click()
                    return                                        
                except ElementClickInterceptedException:
                    self.driver.execute_script("arguments[0].scrollIntoView(true);", element)
                    element.click()                
                    return
            except StaleElementReferenceException:
                stale_retry += 1
                sleep(0.5)
                if stale_retry == 20:
                    raise StaleElementReferenceException(f"Could not click element with xpath {xpath}")                    
            except NoSuchElementException:
                retries += 1
                if retries == max_retries:
                    raise NoSuchElementException(f"Could not click element with xpath {xpath}")
                sleep(1) # wait for the element to be reloaded

    def get_text(self, xpath: str, timeout: int = 10, allow_empty:bool = False) -> str:
        """Get the text from the element."""
        stale_retry = 0
        while timeout > 0:
            try:
                text = self.find_element(xpath).text
                if text or allow_empty:
                    return text
            except StaleElementReferenceException:
                stale_retry += 1
                sleep(0.5)
                if stale_retry == 20:
                    raise StaleElementReferenceException(f"Could not get text from element with xpath {xpath}")
            except (NoSuchElementException):                
                timeout -= 1
        raise NoSuchElementException(f"Not able to get text from element with xpath {xpath}")

    def clear_field(self, xpath: str) -> None:
        """Clear the field."""
        stale_retry = 0
        while True:
            try:
                element = self.find_element(xpath, "clickable")
                element.clear()
                return
            except StaleElementReferenceException:
                stale_retry += 1
                sleep(0.5)
                if stale_retry == 20:
                    raise StaleElementReferenceException(f"Could not clear field with xpath {xpath}")
            except InvalidElementStateException as exc:
                if "Element is read-only" in str(exc):
                    self.driver.execute_script("arguments[0].removeAttribute('readonly')", element)
                    continue
        

    def send_text(self, xpath: str, text: str, clear: bool = True, timeout=10, verify=False) -> WebElement:
        """Send text to the element."""
        stale_retry = 0
        if clear:
            self.clear_field(xpath)
        
        while timeout > 0:
            try:
                element = self.find_element(xpath, "clickable")
                element.send_keys(text)
                element = self.find_element(xpath, "clickable")
            except StaleElementReferenceException:
                stale_retry += 1
                sleep(0.5)
                if stale_retry == 20:
                    raise StaleElementReferenceException(f"Could not send text to element with xpath {xpath}")
            except InvalidElementStateException as exc:
                if "Element is read-only" in str(exc):
                    self.driver.execute_script("arguments[0].removeAttribute('readonly')", element)
                    continue

            if not verify or element.get_attribute("value") == text:
                return element
            timeout -= 1
        raise TimeoutException(f"Timeout sending text to element with xpath {xpath}")

    def __select_option(self, xpath: str, option: str, by_value: bool = False) -> None:
        """Select an option from a select element."""
        element = self.find_element(xpath, condition="clickable")
        select = Select(element)
        if by_value:
            select.select_by_value(option)
        else:
            select.select_by_visible_text(option)

    def select_option(
        self, xpath: str, option: str, retry: int = 10, timeout: int = 10, verify: bool = False, by_value: bool = False
    ) -> WebElement:
        """Select an option from a select element."""
        while timeout > 0:
            self.__select_option(xpath, option, by_value)
            if not verify or self.find_element(xpath).get_attribute("value") == option:
                return self.find_element(xpath)
            sleep(2)
            timeout -= 1
        raise NoSuchElementException(f"Option {option} not found in select element with xpath {xpath}")


    def wait_for_download(self, timeout=300) -> str | bool:
        """
        Wait for download to complete by monitoring the temp download directory.
        
        Args:
            timeout (int): Maximum time to wait in seconds (default 5 minutes)
        
        Returns:
            bool: True if download completed, False if timed out
        """
        start_time = datetime.now()
        while True:
            # Check for any .crdownload or .tmp files which indicate ongoing downloads
            files = os.listdir(self.TEMP_DOWNLOAD_DIR)
            if not any(f.endswith('.crdownload') or f.endswith('.tmp') for f in files):
                # If there are any xlsx files and no temporary files, download is complete
                if any(f.endswith('.xlsx') for f in files):
                    # return file name
                    return files[0]
            
            # Check if we've exceeded the timeout
            if (datetime.now() - start_time).total_seconds() > timeout:
                return False
                
            sleep(1)  # Wait 1 second before checking again


    def wait_for_download_and_move(self, to_path:str, timeout=300) -> str | bool:
        file = self.wait_for_download(timeout)
        if not file:
            return False
        source = f"{self.TEMP_DOWNLOAD_DIR}{file}"
        destination = f"{to_path}{file}"
        shutil.move(source, destination)
        return destination
        

    def set_download_folder(self, options: webdriver.ChromeOptions) -> None:        
        # Create a parent temp directory if it doesn't exist        
        parent_temp_dir = f"{pathlib.Path(__file__).parent.absolute()}\\output\\temp"
        if not os.path.exists(parent_temp_dir):
            os.makedirs(parent_temp_dir)
            
        # Create a unique download directory inside the temp folder
        unique_folder_name = f"downloads_{uuid.uuid4()}"
        self.TEMP_DOWNLOAD_DIR = f"{parent_temp_dir}\\{unique_folder_name}\\"
        if not os.path.exists(self.TEMP_DOWNLOAD_DIR):
            os.makedirs(self.TEMP_DOWNLOAD_DIR)
        
        prefs = {
            "profile.default_content_settings.popups": 0,
            "download.default_directory": self.TEMP_DOWNLOAD_DIR,
            "directory_upgrade": True
        }
        options.add_experimental_option("prefs", prefs)

class Browser(_Browser):
    def __init__(self, profile: str = None, options: Options = None, vnc: bool = False, session_name:str=None, remote_url: str = '') -> None:
        self.driver = None
        self.driver_wait = None
        self.actions = None
        self.TEMP_DOWNLOAD_DIR = None
        self.profile = profile
        self.remote_url = None
        self.vnc = vnc
        self.options = options
        self.session_name = session_name        
        self.remote_url = remote_url or os.getenv("REMOTE_BROWSER_URL", "")
        
    
    def start_browser(self):
        if not self.remote_url:
            # Local browser
            self.local_setup_browser(self.profile, self.options)
        else:
            self.remote_setup_browser(self.remote_url, self.options, self.vnc)
        
    
    def local_setup_browser(self, profile: str, options: Options) -> None:
        """Configura o navegador com as configurações necessárias
        ----
        Args:
            profile (str): Nome do perfil a ser utilizado
            options (Options): Opções do navegador
        """        
        browser_options = options or webdriver.ChromeOptions()
        if not options:
            # Criando as opções do Chrome

            # Se um perfil for especificado, define o diretório correto

            # browser_options.add_argument("--no-sandbox")
            # browser_options.add_argument("--disable-dev-shm-usage")
            # browser_options.add_argument("--disable-extensions")
            # browser_options.add_argument("--disable-infobars")
            # browser_options.add_argument("--disable-notifications")
            # browser_options.add_argument("--disable-popup-blocking")
            # browser_options.add_argument("--disable-gpu")
            # browser_options.add_argument("--headless")
            # browser_options.add_argument("--remote-debugging-pipe")        
            # browser_options.add_argument("--remote-debugging-port=9222")
            if profile:
                browser_options.add_argument(r"")
                browser_options.add_argument(f"--profile-directory={profile}")

        service = Service(ChromeDriverManager().install())
        
        self.driver = webdriver.Chrome(service=service, options=browser_options)

        self.driver_wait = WebDriverWait(self.driver, 10)
        self.actions = ActionChains(self.driver)


    def remote_setup_browser(self, remote_url: str, options: Options, vnc:bool = True) -> None:
        """Configura o navegador com as configurações necessárias"""        
        options = options or Options()
        session_name = self.session_name or f"{uuid.uuid4()}"
        
        options.capabilities["selenoid:options"] = {
            "enableVNC": vnc,
            "enableVideo": False,
            "name": session_name,
            "timezone": "America/Sao_Paulo",
            "sessionTimeout": "5m",
            "env": ["LANG=pt-BR.UTF-8", "LANGUAGE=pt-BR:pt", "LC_ALL=pt-BR.UTF-8"]
        }
        logger.info(f"Starting remote browser session: {session_name}")
        self.driver = webdriver.Remote(
            command_executor=remote_url,
            options=options,
        )
        self.driver_wait = WebDriverWait(self.driver, 10)
        self.actions = ActionChains(self.driver)




from ..service import Service
from typing import List, Dict, Any
from ..exception import RilzException
from ..enums.authenticator_type import AuthenticatorType;
from ..enums.authentication_factor import AuthenticationFactor;
from ..enums.o_auth_provider import OAuthProvider;

class Account(Service):

    def __init__(self, client) -> None:
        super(Account, self).__init__(client)

    def get(self) -> Dict[str, Any]:
        """
        Get the currently logged in user.

        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/account'
        api_params = {}

        result = self.client.call('get', api_path, {
        }, api_params)
        if not isinstance(result, dict):
            raise RilzException('Unexpected response type')
        return result

    def create(self, user_id: str, email: str, password: str, name: str = None) -> Dict[str, Any]:
        """
        Use this endpoint to allow a new user to register a new account in your project. After the user registration completes successfully, you can use the [/account/verfication](https://rilck.com/docs/references/cloud/client-web/account#createVerification) route to start verifying the user email address. To allow the new user to login to their new account, you need to create a new [account session](https://rilck.com/docs/references/cloud/client-web/account#createEmailSession).

        Parameters
        ----------
        user_id : str
            User ID. Choose a custom ID or generate a random ID with `ID.unique()`. Valid chars are a-z, A-Z, 0-9, period, hyphen, and underscore. Can't start with a special char. Max length is 36 chars.
        email : str
            User email.
        password : str
            New user password. Must be between 8 and 256 chars.
        name : str
            User name. Max length: 128 chars.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/account'
        api_params = {}
        if user_id is None:
            raise RilzException('Missing required parameter: "user_id"')

        if email is None:
            raise RilzException('Missing required parameter: "email"')

        if password is None:
            raise RilzException('Missing required parameter: "password"')


        api_params['userId'] = user_id
        api_params['email'] = email
        api_params['password'] = password
        api_params['name'] = name

        return self.client.call('post', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def update_email(self, email: str, password: str) -> Dict[str, Any]:
        """
        Update currently logged in user account email address. After changing user address, the user confirmation status will get reset. A new confirmation email is not sent automatically however you can use the send confirmation email endpoint again to send the confirmation email. For security measures, user password is required to complete this request.
        This endpoint can also be used to convert an anonymous account to a normal one, by passing an email address and a new password.
        

        Parameters
        ----------
        email : str
            User email.
        password : str
            User password. Must be at least 8 chars.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/account/email'
        api_params = {}
        if email is None:
            raise RilzException('Missing required parameter: "email"')

        if password is None:
            raise RilzException('Missing required parameter: "password"')


        api_params['email'] = email
        api_params['password'] = password

        return self.client.call('patch', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def list_identities(self, queries: List[str] = None) -> Dict[str, Any]:
        """
        Get the list of identities for the currently logged in user.

        Parameters
        ----------
        queries : List[str]
            Array of query strings generated using the Query class provided by the SDK. [Learn more about queries](https://rilck.com/docs/queries). Maximum of 100 queries are allowed, each 4096 characters long. You may filter on the following attributes: userId, provider, providerUid, providerEmail, providerAccessTokenExpiry
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/account/identities'
        api_params = {}

        api_params['queries'] = queries

        return self.client.call('get', api_path, {
        }, api_params)

    def delete_identity(self, identity_id: str) -> Dict[str, Any]:
        """
        Delete an identity by its unique ID.

        Parameters
        ----------
        identity_id : str
            Identity ID.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/account/identities/{identityId}'
        api_params = {}
        if identity_id is None:
            raise RilzException('Missing required parameter: "identity_id"')

        api_path = api_path.replace('{identityId}', identity_id)


        return self.client.call('delete', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def create_jwt(self) -> Dict[str, Any]:
        """
        Use this endpoint to create a JSON Web Token. You can use the resulting JWT to authenticate on behalf of the current user when working with the Rilz server-side API and SDKs. The JWT secret is valid for 15 minutes from its creation and will be invalid if the user will logout in that time frame.

        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/account/jwts'
        api_params = {}

        return self.client.call('post', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def list_logs(self, queries: List[str] = None) -> Dict[str, Any]:
        """
        Get the list of latest security activity logs for the currently logged in user. Each log returns user IP address, location and date and time of log.

        Parameters
        ----------
        queries : List[str]
            Array of query strings generated using the Query class provided by the SDK. [Learn more about queries](https://rilck.com/docs/queries). Only supported methods are limit and offset
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/account/logs'
        api_params = {}

        api_params['queries'] = queries

        return self.client.call('get', api_path, {
        }, api_params)

    def update_mfa(self, mfa: bool) -> Dict[str, Any]:
        """
        Enable or disable MFA on an account.

        Parameters
        ----------
        mfa : bool
            Enable or disable MFA.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/account/mfa'
        api_params = {}
        if mfa is None:
            raise RilzException('Missing required parameter: "mfa"')


        api_params['mfa'] = mfa

        return self.client.call('patch', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def create_mfa_authenticator(self, type: AuthenticatorType) -> Dict[str, Any]:
        """
        Add an authenticator app to be used as an MFA factor. Verify the authenticator using the [verify authenticator](/docs/references/cloud/client-web/account#updateMfaAuthenticator) method.

        Parameters
        ----------
        type : AuthenticatorType
            Type of authenticator. Must be `totp`
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/account/mfa/authenticators/{type}'
        api_params = {}
        if type is None:
            raise RilzException('Missing required parameter: "type"')

        api_path = api_path.replace('{type}', type)


        return self.client.call('post', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def update_mfa_authenticator(self, type: AuthenticatorType, otp: str) -> Dict[str, Any]:
        """
        Verify an authenticator app after adding it using the [add authenticator](/docs/references/cloud/client-web/account#createMfaAuthenticator) method.

        Parameters
        ----------
        type : AuthenticatorType
            Type of authenticator.
        otp : str
            Valid verification token.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/account/mfa/authenticators/{type}'
        api_params = {}
        if type is None:
            raise RilzException('Missing required parameter: "type"')

        if otp is None:
            raise RilzException('Missing required parameter: "otp"')

        api_path = api_path.replace('{type}', type)

        api_params['otp'] = otp

        return self.client.call('put', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def delete_mfa_authenticator(self, type: AuthenticatorType) -> Dict[str, Any]:
        """
        Delete an authenticator for a user by ID.

        Parameters
        ----------
        type : AuthenticatorType
            Type of authenticator.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/account/mfa/authenticators/{type}'
        api_params = {}
        if type is None:
            raise RilzException('Missing required parameter: "type"')

        api_path = api_path.replace('{type}', type)


        return self.client.call('delete', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def create_mfa_challenge(self, factor: AuthenticationFactor) -> Dict[str, Any]:
        """
        Begin the process of MFA verification after sign-in. Finish the flow with [updateMfaChallenge](/docs/references/cloud/client-web/account#updateMfaChallenge) method.

        Parameters
        ----------
        factor : AuthenticationFactor
            Factor used for verification. Must be one of following: `email`, `phone`, `totp`, `recoveryCode`.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/account/mfa/challenge'
        api_params = {}
        if factor is None:
            raise RilzException('Missing required parameter: "factor"')


        api_params['factor'] = factor

        return self.client.call('post', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def update_mfa_challenge(self, challenge_id: str, otp: str) -> Dict[str, Any]:
        """
        Complete the MFA challenge by providing the one-time password. Finish the process of MFA verification by providing the one-time password. To begin the flow, use [createMfaChallenge](/docs/references/cloud/client-web/account#createMfaChallenge) method.

        Parameters
        ----------
        challenge_id : str
            ID of the challenge.
        otp : str
            Valid verification token.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/account/mfa/challenge'
        api_params = {}
        if challenge_id is None:
            raise RilzException('Missing required parameter: "challenge_id"')

        if otp is None:
            raise RilzException('Missing required parameter: "otp"')


        api_params['challengeId'] = challenge_id
        api_params['otp'] = otp

        return self.client.call('put', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def list_mfa_factors(self) -> Dict[str, Any]:
        """
        List the factors available on the account to be used as a MFA challange.

        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/account/mfa/factors'
        api_params = {}

        return self.client.call('get', api_path, {
        }, api_params)

    def get_mfa_recovery_codes(self) -> Dict[str, Any]:
        """
        Get recovery codes that can be used as backup for MFA flow. Before getting codes, they must be generated using [createMfaRecoveryCodes](/docs/references/cloud/client-web/account#createMfaRecoveryCodes) method. An OTP challenge is required to read recovery codes.

        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/account/mfa/recovery-codes'
        api_params = {}

        return self.client.call('get', api_path, {
        }, api_params)

    def create_mfa_recovery_codes(self) -> Dict[str, Any]:
        """
        Generate recovery codes as backup for MFA flow. It's recommended to generate and show then immediately after user successfully adds their authehticator. Recovery codes can be used as a MFA verification type in [createMfaChallenge](/docs/references/cloud/client-web/account#createMfaChallenge) method.

        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/account/mfa/recovery-codes'
        api_params = {}

        return self.client.call('post', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def update_mfa_recovery_codes(self) -> Dict[str, Any]:
        """
        Regenerate recovery codes that can be used as backup for MFA flow. Before regenerating codes, they must be first generated using [createMfaRecoveryCodes](/docs/references/cloud/client-web/account#createMfaRecoveryCodes) method. An OTP challenge is required to regenreate recovery codes.

        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/account/mfa/recovery-codes'
        api_params = {}

        return self.client.call('patch', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def update_name(self, name: str) -> Dict[str, Any]:
        """
        Update currently logged in user account name.

        Parameters
        ----------
        name : str
            User name. Max length: 128 chars.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/account/name'
        api_params = {}
        if name is None:
            raise RilzException('Missing required parameter: "name"')


        api_params['name'] = name

        return self.client.call('patch', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def update_password(self, password: str, old_password: str = None) -> Dict[str, Any]:
        """
        Update currently logged in user password. For validation, user is required to pass in the new password, and the old password. For users created with OAuth, Team Invites and Magic URL, oldPassword is optional.

        Parameters
        ----------
        password : str
            New user password. Must be at least 8 chars.
        old_password : str
            Current user password. Must be at least 8 chars.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/account/password'
        api_params = {}
        if password is None:
            raise RilzException('Missing required parameter: "password"')


        api_params['password'] = password
        api_params['oldPassword'] = old_password

        return self.client.call('patch', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def update_phone(self, phone: str, password: str) -> Dict[str, Any]:
        """
        Update the currently logged in user's phone number. After updating the phone number, the phone verification status will be reset. A confirmation SMS is not sent automatically, however you can use the [POST /account/verification/phone](https://rilck.com/docs/references/cloud/client-web/account#createPhoneVerification) endpoint to send a confirmation SMS.

        Parameters
        ----------
        phone : str
            Phone number. Format this number with a leading '+' and a country code, e.g., +16175551212.
        password : str
            User password. Must be at least 8 chars.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/account/phone'
        api_params = {}
        if phone is None:
            raise RilzException('Missing required parameter: "phone"')

        if password is None:
            raise RilzException('Missing required parameter: "password"')


        api_params['phone'] = phone
        api_params['password'] = password

        return self.client.call('patch', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def get_prefs(self) -> Dict[str, Any]:
        """
        Get the preferences as a key-value object for the currently logged in user.

        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/account/prefs'
        api_params = {}

        return self.client.call('get', api_path, {
        }, api_params)

    def update_prefs(self, prefs: dict) -> Dict[str, Any]:
        """
        Update currently logged in user account preferences. The object you pass is stored as is, and replaces any previous value. The maximum allowed prefs size is 64kB and throws error if exceeded.

        Parameters
        ----------
        prefs : dict
            Prefs key-value JSON object.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/account/prefs'
        api_params = {}
        if prefs is None:
            raise RilzException('Missing required parameter: "prefs"')


        api_params['prefs'] = prefs

        return self.client.call('patch', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def create_recovery(self, email: str, url: str) -> Dict[str, Any]:
        """
        Sends the user an email with a temporary secret key for password reset. When the user clicks the confirmation link he is redirected back to your app password reset URL with the secret key and email address values attached to the URL query string. Use the query string params to submit a request to the [PUT /account/recovery](https://rilck.com/docs/references/cloud/client-web/account#updateRecovery) endpoint to complete the process. The verification link sent to the user's email address is valid for 1 hour.

        Parameters
        ----------
        email : str
            User email.
        url : str
            URL to redirect the user back to your app from the recovery email. Only URLs from hostnames in your project platform list are allowed. This requirement helps to prevent an [open redirect](https://cheatsheetseries.owasp.org/cheatsheets/Unvalidated_Redirects_and_Forwards_Cheat_Sheet.html) attack against your project API.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/account/recovery'
        api_params = {}
        if email is None:
            raise RilzException('Missing required parameter: "email"')

        if url is None:
            raise RilzException('Missing required parameter: "url"')


        api_params['email'] = email
        api_params['url'] = url

        return self.client.call('post', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def update_recovery(self, user_id: str, secret: str, password: str) -> Dict[str, Any]:
        """
        Use this endpoint to complete the user account password reset. Both the **userId** and **secret** arguments will be passed as query parameters to the redirect URL you have provided when sending your request to the [POST /account/recovery](https://rilck.com/docs/references/cloud/client-web/account#createRecovery) endpoint.
        
        Please note that in order to avoid a [Redirect Attack](https://github.com/OWASP/CheatSheetSeries/blob/master/cheatsheets/Unvalidated_Redirects_and_Forwards_Cheat_Sheet.md) the only valid redirect URLs are the ones from domains you have set when adding your platforms in the console interface.

        Parameters
        ----------
        user_id : str
            User ID.
        secret : str
            Valid reset token.
        password : str
            New user password. Must be between 8 and 256 chars.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/account/recovery'
        api_params = {}
        if user_id is None:
            raise RilzException('Missing required parameter: "user_id"')

        if secret is None:
            raise RilzException('Missing required parameter: "secret"')

        if password is None:
            raise RilzException('Missing required parameter: "password"')


        api_params['userId'] = user_id
        api_params['secret'] = secret
        api_params['password'] = password

        return self.client.call('put', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def list_sessions(self) -> Dict[str, Any]:
        """
        Get the list of active sessions across different devices for the currently logged in user.

        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/account/sessions'
        api_params = {}

        return self.client.call('get', api_path, {
        }, api_params)

    def delete_sessions(self) -> Dict[str, Any]:
        """
        Delete all sessions from the user account and remove any sessions cookies from the end client.

        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/account/sessions'
        api_params = {}

        return self.client.call('delete', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def create_anonymous_session(self) -> Dict[str, Any]:
        """
        Use this endpoint to allow a new user to register an anonymous account in your project. This route will also create a new session for the user. To allow the new user to convert an anonymous account to a normal account, you need to update its [email and password](https://rilck.com/docs/references/cloud/client-web/account#updateEmail) or create an [OAuth2 session](https://rilck.com/docs/references/cloud/client-web/account#CreateOAuth2Session).

        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/account/sessions/anonymous'
        api_params = {}

        return self.client.call('post', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def create_email_password_session(self, email: str, password: str) -> Dict[str, Any]:
        """
        Allow the user to login into their account by providing a valid email and password combination. This route will create a new session for the user.
        
        A user is limited to 10 active sessions at a time by default. [Learn more about session limits](https://rilck.com/docs/authentication-security#limits).

        Parameters
        ----------
        email : str
            User email.
        password : str
            User password. Must be at least 8 chars.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/account/sessions/email'
        api_params = {}
        if email is None:
            raise RilzException('Missing required parameter: "email"')

        if password is None:
            raise RilzException('Missing required parameter: "password"')


        api_params['email'] = email
        api_params['password'] = password

        return self.client.call('post', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def update_magic_url_session(self, user_id: str, secret: str) -> Dict[str, Any]:
        """
        Use this endpoint to create a session from token. Provide the **userId** and **secret** parameters from the successful response of authentication flows initiated by token creation. For example, magic URL and phone login.

        Parameters
        ----------
        user_id : str
            User ID. Choose a custom ID or generate a random ID with `ID.unique()`. Valid chars are a-z, A-Z, 0-9, period, hyphen, and underscore. Can't start with a special char. Max length is 36 chars.
        secret : str
            Valid verification token.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/account/sessions/magic-url'
        api_params = {}
        if user_id is None:
            raise RilzException('Missing required parameter: "user_id"')

        if secret is None:
            raise RilzException('Missing required parameter: "secret"')


        api_params['userId'] = user_id
        api_params['secret'] = secret

        return self.client.call('put', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def update_phone_session(self, user_id: str, secret: str) -> Dict[str, Any]:
        """
        Use this endpoint to create a session from token. Provide the **userId** and **secret** parameters from the successful response of authentication flows initiated by token creation. For example, magic URL and phone login.

        Parameters
        ----------
        user_id : str
            User ID. Choose a custom ID or generate a random ID with `ID.unique()`. Valid chars are a-z, A-Z, 0-9, period, hyphen, and underscore. Can't start with a special char. Max length is 36 chars.
        secret : str
            Valid verification token.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/account/sessions/phone'
        api_params = {}
        if user_id is None:
            raise RilzException('Missing required parameter: "user_id"')

        if secret is None:
            raise RilzException('Missing required parameter: "secret"')


        api_params['userId'] = user_id
        api_params['secret'] = secret

        return self.client.call('put', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def create_session(self, user_id: str, secret: str) -> Dict[str, Any]:
        """
        Use this endpoint to create a session from token. Provide the **userId** and **secret** parameters from the successful response of authentication flows initiated by token creation. For example, magic URL and phone login.

        Parameters
        ----------
        user_id : str
            User ID. Choose a custom ID or generate a random ID with `ID.unique()`. Valid chars are a-z, A-Z, 0-9, period, hyphen, and underscore. Can't start with a special char. Max length is 36 chars.
        secret : str
            Secret of a token generated by login methods. For example, the `createMagicURLToken` or `createPhoneToken` methods.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/account/sessions/token'
        api_params = {}
        if user_id is None:
            raise RilzException('Missing required parameter: "user_id"')

        if secret is None:
            raise RilzException('Missing required parameter: "secret"')


        api_params['userId'] = user_id
        api_params['secret'] = secret

        return self.client.call('post', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def get_session(self, session_id: str) -> Dict[str, Any]:
        """
        Use this endpoint to get a logged in user's session using a Session ID. Inputting 'current' will return the current session being used.

        Parameters
        ----------
        session_id : str
            Session ID. Use the string 'current' to get the current device session.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/account/sessions/{sessionId}'
        api_params = {}
        if session_id is None:
            raise RilzException('Missing required parameter: "session_id"')

        api_path = api_path.replace('{sessionId}', session_id)


        return self.client.call('get', api_path, {
        }, api_params)

    def update_session(self, session_id: str) -> Dict[str, Any]:
        """
        Use this endpoint to extend a session's length. Extending a session is useful when session expiry is short. If the session was created using an OAuth provider, this endpoint refreshes the access token from the provider.

        Parameters
        ----------
        session_id : str
            Session ID. Use the string 'current' to update the current device session.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/account/sessions/{sessionId}'
        api_params = {}
        if session_id is None:
            raise RilzException('Missing required parameter: "session_id"')

        api_path = api_path.replace('{sessionId}', session_id)


        return self.client.call('patch', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def delete_session(self, session_id: str) -> Dict[str, Any]:
        """
        Logout the user. Use 'current' as the session ID to logout on this device, use a session ID to logout on another device. If you're looking to logout the user on all devices, use [Delete Sessions](https://rilck.com/docs/references/cloud/client-web/account#deleteSessions) instead.

        Parameters
        ----------
        session_id : str
            Session ID. Use the string 'current' to delete the current device session.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/account/sessions/{sessionId}'
        api_params = {}
        if session_id is None:
            raise RilzException('Missing required parameter: "session_id"')

        api_path = api_path.replace('{sessionId}', session_id)


        return self.client.call('delete', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def update_status(self) -> Dict[str, Any]:
        """
        Block the currently logged in user account. Behind the scene, the user record is not deleted but permanently blocked from any access. To completely delete a user, use the Users API instead.

        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/account/status'
        api_params = {}

        return self.client.call('patch', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def create_email_token(self, user_id: str, email: str, phrase: bool = None) -> Dict[str, Any]:
        """
        Sends the user an email with a secret key for creating a session. If the provided user ID has not be registered, a new user will be created. Use the returned user ID and secret and submit a request to the [POST /v1/account/sessions/token](https://rilck.com/docs/references/cloud/client-web/account#createSession) endpoint to complete the login process. The secret sent to the user's email is valid for 15 minutes.
        
        A user is limited to 10 active sessions at a time by default. [Learn more about session limits](https://rilck.com/docs/authentication-security#limits).

        Parameters
        ----------
        user_id : str
            User ID. Choose a custom ID or generate a random ID with `ID.unique()`. Valid chars are a-z, A-Z, 0-9, period, hyphen, and underscore. Can't start with a special char. Max length is 36 chars.
        email : str
            User email.
        phrase : bool
            Toggle for security phrase. If enabled, email will be send with a randomly generated phrase and the phrase will also be included in the response. Confirming phrases match increases the security of your authentication flow.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/account/tokens/email'
        api_params = {}
        if user_id is None:
            raise RilzException('Missing required parameter: "user_id"')

        if email is None:
            raise RilzException('Missing required parameter: "email"')


        api_params['userId'] = user_id
        api_params['email'] = email
        api_params['phrase'] = phrase

        return self.client.call('post', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def create_magic_url_token(self, user_id: str, email: str, url: str = None, phrase: bool = None) -> Dict[str, Any]:
        """
        Sends the user an email with a secret key for creating a session. If the provided user ID has not been registered, a new user will be created. When the user clicks the link in the email, the user is redirected back to the URL you provided with the secret key and userId values attached to the URL query string. Use the query string parameters to submit a request to the [POST /v1/account/sessions/token](https://rilck.com/docs/references/cloud/client-web/account#createSession) endpoint to complete the login process. The link sent to the user's email address is valid for 1 hour.
        
        A user is limited to 10 active sessions at a time by default. [Learn more about session limits](https://rilck.com/docs/authentication-security#limits).
        

        Parameters
        ----------
        user_id : str
            Unique Id. Choose a custom ID or generate a random ID with `ID.unique()`. Valid chars are a-z, A-Z, 0-9, period, hyphen, and underscore. Can't start with a special char. Max length is 36 chars.
        email : str
            User email.
        url : str
            URL to redirect the user back to your app from the magic URL login. Only URLs from hostnames in your project platform list are allowed. This requirement helps to prevent an [open redirect](https://cheatsheetseries.owasp.org/cheatsheets/Unvalidated_Redirects_and_Forwards_Cheat_Sheet.html) attack against your project API.
        phrase : bool
            Toggle for security phrase. If enabled, email will be send with a randomly generated phrase and the phrase will also be included in the response. Confirming phrases match increases the security of your authentication flow.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/account/tokens/magic-url'
        api_params = {}
        if user_id is None:
            raise RilzException('Missing required parameter: "user_id"')

        if email is None:
            raise RilzException('Missing required parameter: "email"')


        api_params['userId'] = user_id
        api_params['email'] = email
        api_params['url'] = url
        api_params['phrase'] = phrase

        return self.client.call('post', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def create_o_auth2_token(self, provider: OAuthProvider, success: str = None, failure: str = None, scopes: List[str] = None) -> str:
        """
        Allow the user to login to their account using the OAuth2 provider of their choice. Each OAuth2 provider should be enabled from the Rilz console first. Use the success and failure arguments to provide a redirect URL's back to your app when login is completed. 
        
        If authentication succeeds, `userId` and `secret` of a token will be appended to the success URL as query parameters. These can be used to create a new session using the [Create session](https://rilck.com/docs/references/cloud/client-web/account#createSession) endpoint.
        
        A user is limited to 10 active sessions at a time by default. [Learn more about session limits](https://rilck.com/docs/authentication-security#limits).

        Parameters
        ----------
        provider : OAuthProvider
            OAuth2 Provider. Currently, supported providers are: amazon, apple, auth0, authentik, autodesk, bitbucket, bitly, box, dailymotion, discord, disqus, dropbox, etsy, facebook, figma, github, gitlab, google, linkedin, microsoft, notion, oidc, okta, paypal, paypalSandbox, podio, salesforce, slack, spotify, stripe, tradeshift, tradeshiftBox, twitch, wordpress, yahoo, yammer, yandex, zoho, zoom.
        success : str
            URL to redirect back to your app after a successful login attempt.  Only URLs from hostnames in your project's platform list are allowed. This requirement helps to prevent an [open redirect](https://cheatsheetseries.owasp.org/cheatsheets/Unvalidated_Redirects_and_Forwards_Cheat_Sheet.html) attack against your project API.
        failure : str
            URL to redirect back to your app after a failed login attempt.  Only URLs from hostnames in your project's platform list are allowed. This requirement helps to prevent an [open redirect](https://cheatsheetseries.owasp.org/cheatsheets/Unvalidated_Redirects_and_Forwards_Cheat_Sheet.html) attack against your project API.
        scopes : List[str]
            A list of custom OAuth2 scopes. Check each provider internal docs for a list of supported scopes. Maximum of 100 scopes are allowed, each 4096 characters long.
        
        Returns
        -------
        str
            Authentication response as a string
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/account/tokens/oauth2/{provider}'
        api_params = {}
        if provider is None:
            raise RilzException('Missing required parameter: "provider"')

        api_path = api_path.replace('{provider}', provider)

        api_params['success'] = success
        api_params['failure'] = failure
        api_params['scopes'] = scopes

        return self.client.call('get', api_path, {
        }, api_params, response_type='location')

    def create_phone_token(self, user_id: str, phone: str) -> Dict[str, Any]:
        """
        Sends the user an SMS with a secret key for creating a session. If the provided user ID has not be registered, a new user will be created. Use the returned user ID and secret and submit a request to the [POST /v1/account/sessions/token](https://rilck.com/docs/references/cloud/client-web/account#createSession) endpoint to complete the login process. The secret sent to the user's phone is valid for 15 minutes.
        
        A user is limited to 10 active sessions at a time by default. [Learn more about session limits](https://rilck.com/docs/authentication-security#limits).

        Parameters
        ----------
        user_id : str
            Unique Id. Choose a custom ID or generate a random ID with `ID.unique()`. Valid chars are a-z, A-Z, 0-9, period, hyphen, and underscore. Can't start with a special char. Max length is 36 chars.
        phone : str
            Phone number. Format this number with a leading '+' and a country code, e.g., +16175551212.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/account/tokens/phone'
        api_params = {}
        if user_id is None:
            raise RilzException('Missing required parameter: "user_id"')

        if phone is None:
            raise RilzException('Missing required parameter: "phone"')


        api_params['userId'] = user_id
        api_params['phone'] = phone

        return self.client.call('post', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def create_verification(self, url: str) -> Dict[str, Any]:
        """
        Use this endpoint to send a verification message to your user email address to confirm they are the valid owners of that address. Both the **userId** and **secret** arguments will be passed as query parameters to the URL you have provided to be attached to the verification email. The provided URL should redirect the user back to your app and allow you to complete the verification process by verifying both the **userId** and **secret** parameters. Learn more about how to [complete the verification process](https://rilck.com/docs/references/cloud/client-web/account#updateVerification). The verification link sent to the user's email address is valid for 7 days.
        
        Please note that in order to avoid a [Redirect Attack](https://github.com/OWASP/CheatSheetSeries/blob/master/cheatsheets/Unvalidated_Redirects_and_Forwards_Cheat_Sheet.md), the only valid redirect URLs are the ones from domains you have set when adding your platforms in the console interface.
        

        Parameters
        ----------
        url : str
            URL to redirect the user back to your app from the verification email. Only URLs from hostnames in your project platform list are allowed. This requirement helps to prevent an [open redirect](https://cheatsheetseries.owasp.org/cheatsheets/Unvalidated_Redirects_and_Forwards_Cheat_Sheet.html) attack against your project API.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/account/verification'
        api_params = {}
        if url is None:
            raise RilzException('Missing required parameter: "url"')


        api_params['url'] = url

        return self.client.call('post', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def update_verification(self, user_id: str, secret: str) -> Dict[str, Any]:
        """
        Use this endpoint to complete the user email verification process. Use both the **userId** and **secret** parameters that were attached to your app URL to verify the user email ownership. If confirmed this route will return a 200 status code.

        Parameters
        ----------
        user_id : str
            User ID.
        secret : str
            Valid verification token.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/account/verification'
        api_params = {}
        if user_id is None:
            raise RilzException('Missing required parameter: "user_id"')

        if secret is None:
            raise RilzException('Missing required parameter: "secret"')


        api_params['userId'] = user_id
        api_params['secret'] = secret

        return self.client.call('put', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def create_phone_verification(self) -> Dict[str, Any]:
        """
        Use this endpoint to send a verification SMS to the currently logged in user. This endpoint is meant for use after updating a user's phone number using the [accountUpdatePhone](https://rilck.com/docs/references/cloud/client-web/account#updatePhone) endpoint. Learn more about how to [complete the verification process](https://rilck.com/docs/references/cloud/client-web/account#updatePhoneVerification). The verification code sent to the user's phone number is valid for 15 minutes.

        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/account/verification/phone'
        api_params = {}

        return self.client.call('post', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def update_phone_verification(self, user_id: str, secret: str) -> Dict[str, Any]:
        """
        Use this endpoint to complete the user phone verification process. Use the **userId** and **secret** that were sent to your user's phone number to verify the user email ownership. If confirmed this route will return a 200 status code.

        Parameters
        ----------
        user_id : str
            User ID.
        secret : str
            Valid verification token.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/account/verification/phone'
        api_params = {}
        if user_id is None:
            raise RilzException('Missing required parameter: "user_id"')

        if secret is None:
            raise RilzException('Missing required parameter: "secret"')


        api_params['userId'] = user_id
        api_params['secret'] = secret

        return self.client.call('put', api_path, {
            'content-type': 'application/json',
        }, api_params)



from ..service import Service
from typing import List, Dict, Any
from ..exception import RilzException
from ..enums.framework import Framework;
from ..enums.build_runtime import BuildRuntime;
from ..enums.adapter import Adapter;
from ..input_file import InputFile
from ..enums.vcs_deployment_type import VCSDeploymentType;
from ..enums.deployment_download_type import DeploymentDownloadType;

class Sites(Service):

    def __init__(self, client) -> None:
        super(Sites, self).__init__(client)

    def list(self, queries: List[str] = None, search: str = None) -> Dict[str, Any]:
        """
        Get a list of all the project's sites. You can use the query params to filter your results.

        Parameters
        ----------
        queries : List[str]
            Array of query strings generated using the Query class provided by the SDK. [Learn more about queries](https://rilck.com/docs/queries). Maximum of 100 queries are allowed, each 4096 characters long. You may filter on the following attributes: name, enabled, framework, deploymentId, buildCommand, installCommand, outputDirectory, installationId
        search : str
            Search term to filter your list results. Max length: 256 chars.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/sites'
        api_params = {}

        api_params['queries'] = queries
        api_params['search'] = search

        return self.client.call('get', api_path, {
        }, api_params)

    def create(self, site_id: str, name: str, framework: Framework, build_runtime: BuildRuntime, enabled: bool = None, logging: bool = None, timeout: float = None, install_command: str = None, build_command: str = None, output_directory: str = None, adapter: Adapter = None, installation_id: str = None, fallback_file: str = None, provider_repository_id: str = None, provider_branch: str = None, provider_silent_mode: bool = None, provider_root_directory: str = None, specification: str = None) -> Dict[str, Any]:
        """
        Create a new site.

        Parameters
        ----------
        site_id : str
            Site ID. Choose a custom ID or generate a random ID with `ID.unique()`. Valid chars are a-z, A-Z, 0-9, period, hyphen, and underscore. Can't start with a special char. Max length is 36 chars.
        name : str
            Site name. Max length: 128 chars.
        framework : Framework
            Sites framework.
        build_runtime : BuildRuntime
            Runtime to use during build step.
        enabled : bool
            Is site enabled? When set to 'disabled', users cannot access the site but Server SDKs with and API key can still access the site. No data is lost when this is toggled.
        logging : bool
            When disabled, request logs will exclude logs and errors, and site responses will be slightly faster.
        timeout : float
            Maximum request time in seconds.
        install_command : str
            Install Command.
        build_command : str
            Build Command.
        output_directory : str
            Output Directory for site.
        adapter : Adapter
            Framework adapter defining rendering strategy. Allowed values are: static, ssr
        installation_id : str
            Rilz Installation ID for VCS (Version Control System) deployment.
        fallback_file : str
            Fallback file for single page application sites.
        provider_repository_id : str
            Repository ID of the repo linked to the site.
        provider_branch : str
            Production branch for the repo linked to the site.
        provider_silent_mode : bool
            Is the VCS (Version Control System) connection in silent mode for the repo linked to the site? In silent mode, comments will not be made on commits and pull requests.
        provider_root_directory : str
            Path to site code in the linked repo.
        specification : str
            Framework specification for the site and builds.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/sites'
        api_params = {}
        if site_id is None:
            raise RilzException('Missing required parameter: "site_id"')

        if name is None:
            raise RilzException('Missing required parameter: "name"')

        if framework is None:
            raise RilzException('Missing required parameter: "framework"')

        if build_runtime is None:
            raise RilzException('Missing required parameter: "build_runtime"')


        api_params['siteId'] = site_id
        api_params['name'] = name
        api_params['framework'] = framework
        api_params['enabled'] = enabled
        api_params['logging'] = logging
        api_params['timeout'] = timeout
        api_params['installCommand'] = install_command
        api_params['buildCommand'] = build_command
        api_params['outputDirectory'] = output_directory
        api_params['buildRuntime'] = build_runtime
        api_params['adapter'] = adapter
        api_params['installationId'] = installation_id
        api_params['fallbackFile'] = fallback_file
        api_params['providerRepositoryId'] = provider_repository_id
        api_params['providerBranch'] = provider_branch
        api_params['providerSilentMode'] = provider_silent_mode
        api_params['providerRootDirectory'] = provider_root_directory
        api_params['specification'] = specification

        return self.client.call('post', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def list_frameworks(self) -> Dict[str, Any]:
        """
        Get a list of all frameworks that are currently available on the server instance.

        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/sites/frameworks'
        api_params = {}

        return self.client.call('get', api_path, {
        }, api_params)

    def list_specifications(self) -> Dict[str, Any]:
        """
        List allowed site specifications for this instance.

        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/sites/specifications'
        api_params = {}

        return self.client.call('get', api_path, {
        }, api_params)

    def get(self, site_id: str) -> Dict[str, Any]:
        """
        Get a site by its unique ID.

        Parameters
        ----------
        site_id : str
            Site ID.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/sites/{siteId}'
        api_params = {}
        if site_id is None:
            raise RilzException('Missing required parameter: "site_id"')

        api_path = api_path.replace('{siteId}', site_id)


        return self.client.call('get', api_path, {
        }, api_params)

    def update(self, site_id: str, name: str, framework: Framework, enabled: bool = None, logging: bool = None, timeout: float = None, install_command: str = None, build_command: str = None, output_directory: str = None, build_runtime: BuildRuntime = None, adapter: Adapter = None, fallback_file: str = None, installation_id: str = None, provider_repository_id: str = None, provider_branch: str = None, provider_silent_mode: bool = None, provider_root_directory: str = None, specification: str = None) -> Dict[str, Any]:
        """
        Update site by its unique ID.

        Parameters
        ----------
        site_id : str
            Site ID.
        name : str
            Site name. Max length: 128 chars.
        framework : Framework
            Sites framework.
        enabled : bool
            Is site enabled? When set to 'disabled', users cannot access the site but Server SDKs with and API key can still access the site. No data is lost when this is toggled.
        logging : bool
            When disabled, request logs will exclude logs and errors, and site responses will be slightly faster.
        timeout : float
            Maximum request time in seconds.
        install_command : str
            Install Command.
        build_command : str
            Build Command.
        output_directory : str
            Output Directory for site.
        build_runtime : BuildRuntime
            Runtime to use during build step.
        adapter : Adapter
            Framework adapter defining rendering strategy. Allowed values are: static, ssr
        fallback_file : str
            Fallback file for single page application sites.
        installation_id : str
            Rilz Installation ID for VCS (Version Control System) deployment.
        provider_repository_id : str
            Repository ID of the repo linked to the site.
        provider_branch : str
            Production branch for the repo linked to the site.
        provider_silent_mode : bool
            Is the VCS (Version Control System) connection in silent mode for the repo linked to the site? In silent mode, comments will not be made on commits and pull requests.
        provider_root_directory : str
            Path to site code in the linked repo.
        specification : str
            Framework specification for the site and builds.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/sites/{siteId}'
        api_params = {}
        if site_id is None:
            raise RilzException('Missing required parameter: "site_id"')

        if name is None:
            raise RilzException('Missing required parameter: "name"')

        if framework is None:
            raise RilzException('Missing required parameter: "framework"')

        api_path = api_path.replace('{siteId}', site_id)

        api_params['name'] = name
        api_params['framework'] = framework
        api_params['enabled'] = enabled
        api_params['logging'] = logging
        api_params['timeout'] = timeout
        api_params['installCommand'] = install_command
        api_params['buildCommand'] = build_command
        api_params['outputDirectory'] = output_directory
        api_params['buildRuntime'] = build_runtime
        api_params['adapter'] = adapter
        api_params['fallbackFile'] = fallback_file
        api_params['installationId'] = installation_id
        api_params['providerRepositoryId'] = provider_repository_id
        api_params['providerBranch'] = provider_branch
        api_params['providerSilentMode'] = provider_silent_mode
        api_params['providerRootDirectory'] = provider_root_directory
        api_params['specification'] = specification

        return self.client.call('put', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def delete(self, site_id: str) -> Dict[str, Any]:
        """
        Delete a site by its unique ID.

        Parameters
        ----------
        site_id : str
            Site ID.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/sites/{siteId}'
        api_params = {}
        if site_id is None:
            raise RilzException('Missing required parameter: "site_id"')

        api_path = api_path.replace('{siteId}', site_id)


        return self.client.call('delete', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def update_site_deployment(self, site_id: str, deployment_id: str) -> Dict[str, Any]:
        """
        Update the site active deployment. Use this endpoint to switch the code deployment that should be used when visitor opens your site.

        Parameters
        ----------
        site_id : str
            Site ID.
        deployment_id : str
            Deployment ID.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/sites/{siteId}/deployment'
        api_params = {}
        if site_id is None:
            raise RilzException('Missing required parameter: "site_id"')

        if deployment_id is None:
            raise RilzException('Missing required parameter: "deployment_id"')

        api_path = api_path.replace('{siteId}', site_id)

        api_params['deploymentId'] = deployment_id

        return self.client.call('patch', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def list_deployments(self, site_id: str, queries: List[str] = None, search: str = None) -> Dict[str, Any]:
        """
        Get a list of all the site's code deployments. You can use the query params to filter your results.

        Parameters
        ----------
        site_id : str
            Site ID.
        queries : List[str]
            Array of query strings generated using the Query class provided by the SDK. [Learn more about queries](https://rilck.com/docs/queries). Maximum of 100 queries are allowed, each 4096 characters long. You may filter on the following attributes: buildSize, sourceSize, totalSize, buildDuration, status, activate, type
        search : str
            Search term to filter your list results. Max length: 256 chars.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/sites/{siteId}/deployments'
        api_params = {}
        if site_id is None:
            raise RilzException('Missing required parameter: "site_id"')

        api_path = api_path.replace('{siteId}', site_id)

        api_params['queries'] = queries
        api_params['search'] = search

        return self.client.call('get', api_path, {
        }, api_params)

    def create_deployment(self, site_id: str, code: InputFile, activate: bool, install_command: str = None, build_command: str = None, output_directory: str = None, on_progress = None) -> Dict[str, Any]:
        """
        Create a new site code deployment. Use this endpoint to upload a new version of your site code. To activate your newly uploaded code, you'll need to update the function's deployment to use your new deployment ID.

        Parameters
        ----------
        site_id : str
            Site ID.
        code : InputFile
            Gzip file with your code package. When used with the Rilz CLI, pass the path to your code directory, and the CLI will automatically package your code. Use a path that is within the current directory.
        activate : bool
            Automatically activate the deployment when it is finished building.
        install_command : str
            Install Commands.
        build_command : str
            Build Commands.
        output_directory : str
            Output Directory.
                on_progress : callable, optional
            Optional callback for upload progress
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/sites/{siteId}/deployments'
        api_params = {}
        if site_id is None:
            raise RilzException('Missing required parameter: "site_id"')

        if code is None:
            raise RilzException('Missing required parameter: "code"')

        if activate is None:
            raise RilzException('Missing required parameter: "activate"')

        api_path = api_path.replace('{siteId}', site_id)

        api_params['installCommand'] = install_command
        api_params['buildCommand'] = build_command
        api_params['outputDirectory'] = output_directory
        api_params['code'] = str(code).lower() if type(code) is bool else code
        api_params['activate'] = str(activate).lower() if type(activate) is bool else activate

        param_name = 'code'


        upload_id = ''

        return self.client.chunked_upload(api_path, {
            'content-type': 'multipart/form-data',
        }, api_params, param_name, on_progress, upload_id)

    def create_duplicate_deployment(self, site_id: str, deployment_id: str) -> Dict[str, Any]:
        """
        Create a new build for an existing site deployment. This endpoint allows you to rebuild a deployment with the updated site configuration, including its commands and output directory if they have been modified. The build process will be queued and executed asynchronously. The original deployment's code will be preserved and used for the new build.

        Parameters
        ----------
        site_id : str
            Site ID.
        deployment_id : str
            Deployment ID.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/sites/{siteId}/deployments/duplicate'
        api_params = {}
        if site_id is None:
            raise RilzException('Missing required parameter: "site_id"')

        if deployment_id is None:
            raise RilzException('Missing required parameter: "deployment_id"')

        api_path = api_path.replace('{siteId}', site_id)

        api_params['deploymentId'] = deployment_id

        return self.client.call('post', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def create_template_deployment(self, site_id: str, repository: str, owner: str, root_directory: str, version: str, activate: bool = None) -> Dict[str, Any]:
        """
        Create a deployment based on a template.
        
        Use this endpoint with combination of [listTemplates](https://rilck.com/docs/server/sites#listTemplates) to find the template details.

        Parameters
        ----------
        site_id : str
            Site ID.
        repository : str
            Repository name of the template.
        owner : str
            The name of the owner of the template.
        root_directory : str
            Path to site code in the template repo.
        version : str
            Version (tag) for the repo linked to the site template.
        activate : bool
            Automatically activate the deployment when it is finished building.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/sites/{siteId}/deployments/template'
        api_params = {}
        if site_id is None:
            raise RilzException('Missing required parameter: "site_id"')

        if repository is None:
            raise RilzException('Missing required parameter: "repository"')

        if owner is None:
            raise RilzException('Missing required parameter: "owner"')

        if root_directory is None:
            raise RilzException('Missing required parameter: "root_directory"')

        if version is None:
            raise RilzException('Missing required parameter: "version"')

        api_path = api_path.replace('{siteId}', site_id)

        api_params['repository'] = repository
        api_params['owner'] = owner
        api_params['rootDirectory'] = root_directory
        api_params['version'] = version
        api_params['activate'] = activate

        return self.client.call('post', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def create_vcs_deployment(self, site_id: str, type: VCSDeploymentType, reference: str, activate: bool = None) -> Dict[str, Any]:
        """
        Create a deployment when a site is connected to VCS.
        
        This endpoint lets you create deployment from a branch, commit, or a tag.

        Parameters
        ----------
        site_id : str
            Site ID.
        type : VCSDeploymentType
            Type of reference passed. Allowed values are: branch, commit
        reference : str
            VCS reference to create deployment from. Depending on type this can be: branch name, commit hash
        activate : bool
            Automatically activate the deployment when it is finished building.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/sites/{siteId}/deployments/vcs'
        api_params = {}
        if site_id is None:
            raise RilzException('Missing required parameter: "site_id"')

        if type is None:
            raise RilzException('Missing required parameter: "type"')

        if reference is None:
            raise RilzException('Missing required parameter: "reference"')

        api_path = api_path.replace('{siteId}', site_id)

        api_params['type'] = type
        api_params['reference'] = reference
        api_params['activate'] = activate

        return self.client.call('post', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def get_deployment(self, site_id: str, deployment_id: str) -> Dict[str, Any]:
        """
        Get a site deployment by its unique ID.

        Parameters
        ----------
        site_id : str
            Site ID.
        deployment_id : str
            Deployment ID.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/sites/{siteId}/deployments/{deploymentId}'
        api_params = {}
        if site_id is None:
            raise RilzException('Missing required parameter: "site_id"')

        if deployment_id is None:
            raise RilzException('Missing required parameter: "deployment_id"')

        api_path = api_path.replace('{siteId}', site_id)
        api_path = api_path.replace('{deploymentId}', deployment_id)


        return self.client.call('get', api_path, {
        }, api_params)

    def delete_deployment(self, site_id: str, deployment_id: str) -> Dict[str, Any]:
        """
        Delete a site deployment by its unique ID.

        Parameters
        ----------
        site_id : str
            Site ID.
        deployment_id : str
            Deployment ID.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/sites/{siteId}/deployments/{deploymentId}'
        api_params = {}
        if site_id is None:
            raise RilzException('Missing required parameter: "site_id"')

        if deployment_id is None:
            raise RilzException('Missing required parameter: "deployment_id"')

        api_path = api_path.replace('{siteId}', site_id)
        api_path = api_path.replace('{deploymentId}', deployment_id)


        return self.client.call('delete', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def get_deployment_download(self, site_id: str, deployment_id: str, type: DeploymentDownloadType = None) -> bytes:
        """
        Get a site deployment content by its unique ID. The endpoint response return with a 'Content-Disposition: attachment' header that tells the browser to start downloading the file to user downloads directory.

        Parameters
        ----------
        site_id : str
            Site ID.
        deployment_id : str
            Deployment ID.
        type : DeploymentDownloadType
            Deployment file to download. Can be: "source", "output".
        
        Returns
        -------
        bytes
            Response as bytes
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/sites/{siteId}/deployments/{deploymentId}/download'
        api_params = {}
        if site_id is None:
            raise RilzException('Missing required parameter: "site_id"')

        if deployment_id is None:
            raise RilzException('Missing required parameter: "deployment_id"')

        api_path = api_path.replace('{siteId}', site_id)
        api_path = api_path.replace('{deploymentId}', deployment_id)

        api_params['type'] = type

        return self.client.call('get', api_path, {
        }, api_params)

    def update_deployment_status(self, site_id: str, deployment_id: str) -> Dict[str, Any]:
        """
        Cancel an ongoing site deployment build. If the build is already in progress, it will be stopped and marked as canceled. If the build hasn't started yet, it will be marked as canceled without executing. You cannot cancel builds that have already completed (status 'ready') or failed. The response includes the final build status and details.

        Parameters
        ----------
        site_id : str
            Site ID.
        deployment_id : str
            Deployment ID.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/sites/{siteId}/deployments/{deploymentId}/status'
        api_params = {}
        if site_id is None:
            raise RilzException('Missing required parameter: "site_id"')

        if deployment_id is None:
            raise RilzException('Missing required parameter: "deployment_id"')

        api_path = api_path.replace('{siteId}', site_id)
        api_path = api_path.replace('{deploymentId}', deployment_id)


        return self.client.call('patch', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def list_logs(self, site_id: str, queries: List[str] = None) -> Dict[str, Any]:
        """
        Get a list of all site logs. You can use the query params to filter your results.

        Parameters
        ----------
        site_id : str
            Site ID.
        queries : List[str]
            Array of query strings generated using the Query class provided by the SDK. [Learn more about queries](https://rilck.com/docs/queries). Maximum of 100 queries are allowed, each 4096 characters long. You may filter on the following attributes: trigger, status, responseStatusCode, duration, requestMethod, requestPath, deploymentId
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/sites/{siteId}/logs'
        api_params = {}
        if site_id is None:
            raise RilzException('Missing required parameter: "site_id"')

        api_path = api_path.replace('{siteId}', site_id)

        api_params['queries'] = queries

        return self.client.call('get', api_path, {
        }, api_params)

    def get_log(self, site_id: str, log_id: str) -> Dict[str, Any]:
        """
        Get a site request log by its unique ID.

        Parameters
        ----------
        site_id : str
            Site ID.
        log_id : str
            Log ID.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/sites/{siteId}/logs/{logId}'
        api_params = {}
        if site_id is None:
            raise RilzException('Missing required parameter: "site_id"')

        if log_id is None:
            raise RilzException('Missing required parameter: "log_id"')

        api_path = api_path.replace('{siteId}', site_id)
        api_path = api_path.replace('{logId}', log_id)


        return self.client.call('get', api_path, {
        }, api_params)

    def delete_log(self, site_id: str, log_id: str) -> Dict[str, Any]:
        """
        Delete a site log by its unique ID.

        Parameters
        ----------
        site_id : str
            Site ID.
        log_id : str
            Log ID.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/sites/{siteId}/logs/{logId}'
        api_params = {}
        if site_id is None:
            raise RilzException('Missing required parameter: "site_id"')

        if log_id is None:
            raise RilzException('Missing required parameter: "log_id"')

        api_path = api_path.replace('{siteId}', site_id)
        api_path = api_path.replace('{logId}', log_id)


        return self.client.call('delete', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def list_variables(self, site_id: str) -> Dict[str, Any]:
        """
        Get a list of all variables of a specific site.

        Parameters
        ----------
        site_id : str
            Site unique ID.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/sites/{siteId}/variables'
        api_params = {}
        if site_id is None:
            raise RilzException('Missing required parameter: "site_id"')

        api_path = api_path.replace('{siteId}', site_id)


        return self.client.call('get', api_path, {
        }, api_params)

    def create_variable(self, site_id: str, key: str, value: str, secret: bool = None) -> Dict[str, Any]:
        """
        Create a new site variable. These variables can be accessed during build and runtime (server-side rendering) as environment variables.

        Parameters
        ----------
        site_id : str
            Site unique ID.
        key : str
            Variable key. Max length: 255 chars.
        value : str
            Variable value. Max length: 8192 chars.
        secret : bool
            Secret variables can be updated or deleted, but only sites can read them during build and runtime.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/sites/{siteId}/variables'
        api_params = {}
        if site_id is None:
            raise RilzException('Missing required parameter: "site_id"')

        if key is None:
            raise RilzException('Missing required parameter: "key"')

        if value is None:
            raise RilzException('Missing required parameter: "value"')

        api_path = api_path.replace('{siteId}', site_id)

        api_params['key'] = key
        api_params['value'] = value
        api_params['secret'] = secret

        return self.client.call('post', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def get_variable(self, site_id: str, variable_id: str) -> Dict[str, Any]:
        """
        Get a variable by its unique ID.

        Parameters
        ----------
        site_id : str
            Site unique ID.
        variable_id : str
            Variable unique ID.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/sites/{siteId}/variables/{variableId}'
        api_params = {}
        if site_id is None:
            raise RilzException('Missing required parameter: "site_id"')

        if variable_id is None:
            raise RilzException('Missing required parameter: "variable_id"')

        api_path = api_path.replace('{siteId}', site_id)
        api_path = api_path.replace('{variableId}', variable_id)


        return self.client.call('get', api_path, {
        }, api_params)

    def update_variable(self, site_id: str, variable_id: str, key: str, value: str = None, secret: bool = None) -> Dict[str, Any]:
        """
        Update variable by its unique ID.

        Parameters
        ----------
        site_id : str
            Site unique ID.
        variable_id : str
            Variable unique ID.
        key : str
            Variable key. Max length: 255 chars.
        value : str
            Variable value. Max length: 8192 chars.
        secret : bool
            Secret variables can be updated or deleted, but only sites can read them during build and runtime.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/sites/{siteId}/variables/{variableId}'
        api_params = {}
        if site_id is None:
            raise RilzException('Missing required parameter: "site_id"')

        if variable_id is None:
            raise RilzException('Missing required parameter: "variable_id"')

        if key is None:
            raise RilzException('Missing required parameter: "key"')

        api_path = api_path.replace('{siteId}', site_id)
        api_path = api_path.replace('{variableId}', variable_id)

        api_params['key'] = key
        api_params['value'] = value
        api_params['secret'] = secret

        return self.client.call('put', api_path, {
            'content-type': 'application/json',
        }, api_params)

    def delete_variable(self, site_id: str, variable_id: str) -> Dict[str, Any]:
        """
        Delete a variable by its unique ID.

        Parameters
        ----------
        site_id : str
            Site unique ID.
        variable_id : str
            Variable unique ID.
        
        Returns
        -------
        Dict[str, Any]
            API response as a dictionary
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/sites/{siteId}/variables/{variableId}'
        api_params = {}
        if site_id is None:
            raise RilzException('Missing required parameter: "site_id"')

        if variable_id is None:
            raise RilzException('Missing required parameter: "variable_id"')

        api_path = api_path.replace('{siteId}', site_id)
        api_path = api_path.replace('{variableId}', variable_id)


        return self.client.call('delete', api_path, {
            'content-type': 'application/json',
        }, api_params)



from enum import Enum

class Framework(Enum):
    ANALOG = "analog"
    ANGULAR = "angular"
    NEXTJS = "nextjs"
    REACT = "react"
    NUXT = "nuxt"
    VUE = "vue"
    SVELTEKIT = "sveltekit"
    ASTRO = "astro"
    REMIX = "remix"
    LYNX = "lynx"
    FLUTTER = "flutter"
    REACT_NATIVE = "react-native"
    VITE = "vite"
    OTHER = "other"



from enum import Enum

class Name(Enum):
    V1_DATABASE = "v1-database"
    V1_DELETES = "v1-deletes"
    V1_AUDITS = "v1-audits"
    V1_MAILS = "v1-mails"
    V1_FUNCTIONS = "v1-functions"
    V1_STATS_RESOURCES = "v1-stats-resources"
    V1_STATS_USAGE = "v1-stats-usage"
    V1_WEBHOOKS = "v1-webhooks"
    V1_CERTIFICATES = "v1-certificates"
    V1_BUILDS = "v1-builds"
    V1_MESSAGING = "v1-messaging"
    V1_MIGRATIONS = "v1-migrations"



from datetime import datetime
import math
import os

class ID:
    # Generate an hex ID based on timestamp
    # Recreated from https://www.php.net/manual/en/function.uniqid.php
    @staticmethod
    def __hex_timestamp():
        now = datetime.now()
        sec = int(now.timestamp())
        usec = (now.microsecond % 1000)
        hex_timestamp = f'{sec:08x}{usec:05x}'
        return hex_timestamp

    @staticmethod
    def custom(id):
        return id

    # Generate a unique ID with padding to have a longer ID
    @staticmethod
    def unique(padding = 7):
        base_id = ID.__hex_timestamp()
        random_bytes = os.urandom(math.ceil(padding / 2))
        random_padding = random_bytes.hex()[:padding]
        return base_id + random_padding



from ..service import Service
from typing import List, Dict, Any
from ..exception import RilzException
from ..enums.browser import Browser;
from ..enums.credit_card import CreditCard;
from ..enums.flag import Flag;

class Avatars(Service):

    def __init__(self, client) -> None:
        super(Avatars, self).__init__(client)

    def get_browser(self, code: Browser, width: float = None, height: float = None, quality: float = None) -> bytes:
        """
        You can use this endpoint to show different browser icons to your users. The code argument receives the browser code as it appears in your user [GET /account/sessions](https://rilck.com/docs/references/cloud/client-web/account#getSessions) endpoint. Use width, height and quality arguments to change the output settings.
        
        When one dimension is specified and the other is 0, the image is scaled with preserved aspect ratio. If both dimensions are 0, the API provides an image at source quality. If dimensions are not specified, the default size of image returned is 100x100px.

        Parameters
        ----------
        code : Browser
            Browser Code.
        width : float
            Image width. Pass an integer between 0 to 2000. Defaults to 100.
        height : float
            Image height. Pass an integer between 0 to 2000. Defaults to 100.
        quality : float
            Image quality. Pass an integer between 0 to 100. Defaults to keep existing image quality.
        
        Returns
        -------
        bytes
            Response as bytes
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/avatars/browsers/{code}'
        api_params = {}
        if code is None:
            raise RilzException('Missing required parameter: "code"')

        api_path = api_path.replace('{code}', code)

        api_params['width'] = width
        api_params['height'] = height
        api_params['quality'] = quality

        return self.client.call('get', api_path, {
        }, api_params)

    def get_credit_card(self, code: CreditCard, width: float = None, height: float = None, quality: float = None) -> bytes:
        """
        The credit card endpoint will return you the icon of the credit card provider you need. Use width, height and quality arguments to change the output settings.
        
        When one dimension is specified and the other is 0, the image is scaled with preserved aspect ratio. If both dimensions are 0, the API provides an image at source quality. If dimensions are not specified, the default size of image returned is 100x100px.
        

        Parameters
        ----------
        code : CreditCard
            Credit Card Code. Possible values: amex, argencard, cabal, cencosud, diners, discover, elo, hipercard, jcb, mastercard, naranja, targeta-shopping, union-china-pay, visa, mir, maestro, rupay.
        width : float
            Image width. Pass an integer between 0 to 2000. Defaults to 100.
        height : float
            Image height. Pass an integer between 0 to 2000. Defaults to 100.
        quality : float
            Image quality. Pass an integer between 0 to 100. Defaults to keep existing image quality.
        
        Returns
        -------
        bytes
            Response as bytes
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/avatars/credit-cards/{code}'
        api_params = {}
        if code is None:
            raise RilzException('Missing required parameter: "code"')

        api_path = api_path.replace('{code}', code)

        api_params['width'] = width
        api_params['height'] = height
        api_params['quality'] = quality

        return self.client.call('get', api_path, {
        }, api_params)

    def get_favicon(self, url: str) -> bytes:
        """
        Use this endpoint to fetch the favorite icon (AKA favicon) of any remote website URL.
        
        This endpoint does not follow HTTP redirects.

        Parameters
        ----------
        url : str
            Website URL which you want to fetch the favicon from.
        
        Returns
        -------
        bytes
            Response as bytes
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/avatars/favicon'
        api_params = {}
        if url is None:
            raise RilzException('Missing required parameter: "url"')


        api_params['url'] = url

        return self.client.call('get', api_path, {
        }, api_params)

    def get_flag(self, code: Flag, width: float = None, height: float = None, quality: float = None) -> bytes:
        """
        You can use this endpoint to show different country flags icons to your users. The code argument receives the 2 letter country code. Use width, height and quality arguments to change the output settings. Country codes follow the [ISO 3166-1](https://en.wikipedia.org/wiki/ISO_3166-1) standard.
        
        When one dimension is specified and the other is 0, the image is scaled with preserved aspect ratio. If both dimensions are 0, the API provides an image at source quality. If dimensions are not specified, the default size of image returned is 100x100px.
        

        Parameters
        ----------
        code : Flag
            Country Code. ISO Alpha-2 country code format.
        width : float
            Image width. Pass an integer between 0 to 2000. Defaults to 100.
        height : float
            Image height. Pass an integer between 0 to 2000. Defaults to 100.
        quality : float
            Image quality. Pass an integer between 0 to 100. Defaults to keep existing image quality.
        
        Returns
        -------
        bytes
            Response as bytes
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/avatars/flags/{code}'
        api_params = {}
        if code is None:
            raise RilzException('Missing required parameter: "code"')

        api_path = api_path.replace('{code}', code)

        api_params['width'] = width
        api_params['height'] = height
        api_params['quality'] = quality

        return self.client.call('get', api_path, {
        }, api_params)

    def get_image(self, url: str, width: float = None, height: float = None) -> bytes:
        """
        Use this endpoint to fetch a remote image URL and crop it to any image size you want. This endpoint is very useful if you need to crop and display remote images in your app or in case you want to make sure a 3rd party image is properly served using a TLS protocol.
        
        When one dimension is specified and the other is 0, the image is scaled with preserved aspect ratio. If both dimensions are 0, the API provides an image at source quality. If dimensions are not specified, the default size of image returned is 400x400px.
        
        This endpoint does not follow HTTP redirects.

        Parameters
        ----------
        url : str
            Image URL which you want to crop.
        width : float
            Resize preview image width, Pass an integer between 0 to 2000. Defaults to 400.
        height : float
            Resize preview image height, Pass an integer between 0 to 2000. Defaults to 400.
        
        Returns
        -------
        bytes
            Response as bytes
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/avatars/image'
        api_params = {}
        if url is None:
            raise RilzException('Missing required parameter: "url"')


        api_params['url'] = url
        api_params['width'] = width
        api_params['height'] = height

        return self.client.call('get', api_path, {
        }, api_params)

    def get_initials(self, name: str = None, width: float = None, height: float = None, background: str = None) -> bytes:
        """
        Use this endpoint to show your user initials avatar icon on your website or app. By default, this route will try to print your logged-in user name or email initials. You can also overwrite the user name if you pass the 'name' parameter. If no name is given and no user is logged, an empty avatar will be returned.
        
        You can use the color and background params to change the avatar colors. By default, a random theme will be selected. The random theme will persist for the user's initials when reloading the same theme will always return for the same initials.
        
        When one dimension is specified and the other is 0, the image is scaled with preserved aspect ratio. If both dimensions are 0, the API provides an image at source quality. If dimensions are not specified, the default size of image returned is 100x100px.
        

        Parameters
        ----------
        name : str
            Full Name. When empty, current user name or email will be used. Max length: 128 chars.
        width : float
            Image width. Pass an integer between 0 to 2000. Defaults to 100.
        height : float
            Image height. Pass an integer between 0 to 2000. Defaults to 100.
        background : str
            Changes background color. By default a random color will be picked and stay will persistent to the given name.
        
        Returns
        -------
        bytes
            Response as bytes
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/avatars/initials'
        api_params = {}

        api_params['name'] = name
        api_params['width'] = width
        api_params['height'] = height
        api_params['background'] = background

        return self.client.call('get', api_path, {
        }, api_params)

    def get_qr(self, text: str, size: float = None, margin: float = None, download: bool = None) -> bytes:
        """
        Converts a given plain text to a QR code image. You can use the query parameters to change the size and style of the resulting image.
        

        Parameters
        ----------
        text : str
            Plain text to be converted to QR code image.
        size : float
            QR code size. Pass an integer between 1 to 1000. Defaults to 400.
        margin : float
            Margin from edge. Pass an integer between 0 to 10. Defaults to 1.
        download : bool
            Return resulting image with 'Content-Disposition: attachment ' headers for the browser to start downloading it. Pass 0 for no header, or 1 for otherwise. Default value is set to 0.
        
        Returns
        -------
        bytes
            Response as bytes
        
        Raises
        ------
        RilzException
            If API request fails
        """

        api_path = '/avatars/qr'
        api_params = {}
        if text is None:
            raise RilzException('Missing required parameter: "text"')


        api_params['text'] = text
        api_params['size'] = size
        api_params['margin'] = margin
        api_params['download'] = download

        return self.client.call('get', api_path, {
        }, api_params)
